/***************************************************************************
 *   Copyright (C) 2009 by Andreas Lindenau   *
 *   DL4JAL@darc.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QtGui>
#include <cmath>
#include <QtGlobal>
#include <QString>

#include "kschreiber.h"
#include "configfile.h"


kschreiber::kschreiber()
{
  ksettings.version = "V1.12";
  ksettings.verdatum = "02.08.2021";
  QString target;

#ifdef Q_WS_WIN
  target = "KLSwin";
#else
  target = "KLSlin";
#endif
#if QT_VERSION >= 0x050000
  infotext = tr("<h2>%1</h2>"
                "<p><B>Version %2</B>\t\t%3 </p>"
                "<p>Entwickelt unter Linux mit QT5</p>"
                "<p>(c) Andreas Lindenau DL4JAL<br>"
                "DL4JAL@t-online.de<br>"
                "http://www.dl4jal.de</p>").arg(target).arg(ksettings.version).arg(ksettings.verdatum);
#else
  infotext = tr("<h2>%1</h2>"
                "<p><B>Version %2</B>\t\t%3 </p>"
                "<p>Entwickelt unter Linux mit QT4</p>"
                "<p>(c) Andreas Lindenau DL4JAL<br>"
                "DL4JAL@t-online.de<br>"
                "http://www.dl4jal.de</p>").arg(target).arg(ksettings.version).arg(ksettings.verdatum);
#endif



  ksettings.setminX(0.0);
  ksettings.setmaxX(10.0);
  ksettings.setnumXTicks(20);
  ksettings.setminY(0.0);
  ksettings.setmaxY(500.0);
  ksettings.setnumYTicks(20);
  ksettings.caption="Caption";
  ksettings.ifontsize=12;

  kadcmax=1023.0;
  kadcoffset=0;

///////////////////////////////////////////////////////////////////////////////
//Schnittstelle zum NWT7 erzeugen erzeugen fuer alle Widgets
//  picmodem = new QextSerialPort("/dev/ttyS0");
//  delete picmodem;
  picmodem = new Modem();
//  picmodem->opentty(weichen.str_tty);
  QObject::connect( picmodem, SIGNAL(setTtyText(QString)), this, SLOT(setAText(QString)));
//  bttyOpen = picmodem->opentty(weichen.str_tty);
///////////////////////////////////////////////////////////////////////////////
  
  testtimer = new QTimer(this);
  QObject::connect(testtimer, SIGNAL(timeout()), this, SLOT(readbfm() ));
  imaxtimer = new QTimer(this);
  QObject::connect(imaxtimer, SIGNAL(timeout()), this, SLOT(readimax() ));
  akurventimer = new QTimer(this);
  QObject::connect(akurventimer, SIGNAL(timeout()), this, SLOT(readakurventimer() ));
  ekurventimer = new QTimer(this);
  QObject::connect(ekurventimer, SIGNAL(timeout()), this, SLOT(readekurventimer() ));

  awidget = new QTabWidget();
  QObject::connect( awidget, SIGNAL(currentChanged(int)), this, SLOT(settabwidget(int)));
  
//  awidget->setTabPosition(QTabWidget::West);
  setCentralWidget(awidget);
  wkschreiber = new QWidget();
  setMinimumSize (600, 480);

  awidget->addTab(wkschreiber, tr("Kennlinien","TabWindow"));
  grafik = new WDisplay(wkschreiber);
  QObject::connect( this, SIGNAL( dbildspeichern(QString)), grafik, SLOT(bildspeichern(QString)));
  QObject::connect(grafik, SIGNAL(sendmwert(QPointF)), SLOT(empfmwert(QPointF)));

  gkurven = new QGroupBox(tr("Kurven"),wkschreiber);
  
  bkurvestart = new QPushButton(gkurven);
  bkurvestart->setText(tr("Start"));
  QObject::connect( bkurvestart, SIGNAL( clicked()), this, SLOT(clickkurvenstart()));
  
  bclearcurven = new QPushButton(gkurven);
  bclearcurven->setText(tr("Kurven loeschen"));
  QObject::connect( bclearcurven, SIGNAL( clicked()), this, SLOT(clickbclearcurven()));
  
  bsethintergrund = new QPushButton(gkurven);
  bsethintergrund->setText(tr("Kurven als Hintergrund"));
  QObject::connect( bsethintergrund, SIGNAL( clicked()), this, SLOT(clickbsethintergrund()));
  
  //labelkurvenanz = new QLabel(tr("Anzahl:","Label"), gkurven);
  //spkurvenanz = new QSpinBox(gkurven);
  //spkurvenanz->setRange(1,5);
  //spkurvenanz->setSingleStep(1);
  //spkurvenanz->setValue(1);
  
  labelui1 = new QLabel("", gkurven);
  boxui1 = new QComboBox(gkurven);
  QObject::connect( boxui1, SIGNAL(currentIndexChanged(int)), this, SLOT(aendernboxui1()));
  
  labelui2 = new QLabel("", gkurven);
  boxui2 = new QComboBox(gkurven);
  QObject::connect( boxui2, SIGNAL(currentIndexChanged(int)), this, SLOT(aendernboxui2(int)));
  
  textedit = new QTextEdit(wkschreiber);
  QObject::connect( textedit, SIGNAL(textChanged()), this, SLOT(setTextEdit()));
  
  gibasis = new QGroupBox(wkschreiber);
  i10ua = new QRadioButton("10 uA", gibasis);
  QObject::connect(i10ua, SIGNAL(toggled(bool)), this, SLOT(aendernIb()));
  i100ua = new QRadioButton("100 uA", gibasis);
  QObject::connect(i100ua, SIGNAL(toggled(bool)), this, SLOT(aendernIb()));
  i1ma = new QRadioButton("1 mA", gibasis);
  QObject::connect(i1ma, SIGNAL(toggled(bool)), this, SLOT(aendernIb()));
  
  i10ua->setChecked(true);

  checkclrcurve = new QCheckBox(tr("auto. Loeschen"), gkurven);

//#############################################################################
//Option Widget
//#############################################################################
  woption = new QWidget();
  awidget->addTab(woption, tr("Optionen / Messreihen","TabWindow"));
  
    
  gcolor = new QGroupBox(tr("Color / Font"),woption);
  bcolorhintergrund = new QPushButton(gcolor);
  bcolorhintergrund->setText(tr("Hintergrund"));
  QObject::connect( bcolorhintergrund, SIGNAL( clicked()), this, SLOT(clickcolorhintergrund()));
  bcolorschrift = new QPushButton(gcolor);
  bcolorschrift->setText(tr("Schrift/Linien"));
  QObject::connect( bcolorschrift, SIGNAL( clicked()), this, SLOT(clickcolorschrift()));
  bcolorueberschrift = new QPushButton(gcolor);
  bcolorueberschrift->setText(tr("Ueberschrift"));
  QObject::connect( bcolorueberschrift, SIGNAL( clicked()), this, SLOT(clickcolorueberschrift()));
  bcolorgitter = new QPushButton(gcolor);
  bcolorgitter->setText(tr("Gitter"));
  QObject::connect( bcolorgitter, SIGNAL( clicked()), this, SLOT(clickcolorgitter()));
  bcolorlinie = new QPushButton(gcolor);
  bcolorlinie->setText(tr("Kurven"));
  QObject::connect( bcolorlinie, SIGNAL( clicked()), this, SLOT(clickcolorlinie()));
  bcolorhlinie = new QPushButton(gcolor);
  bcolorhlinie->setText(tr("Hintergrund-Kurven"));
  QObject::connect( bcolorhlinie, SIGNAL( clicked()), this, SLOT(clickcolorhlinie()));
  sboxliniefsize = new QSpinBox(gcolor);
  sboxliniefsize->setRange(8,20);
  sboxliniefsize->setValue(12);
  QObject::connect( sboxliniefsize, SIGNAL( valueChanged (int)), this, SLOT(setlinieFontSize(int)));
  labelliniefsize = new QLabel(tr("Schriftgroesse Linie rechts","Label"),gcolor);
  labelliniefsize->setAlignment(Qt::AlignCenter);
  sboxliniesize = new QSpinBox(gcolor);
  sboxliniesize->setRange(8,20);
  sboxliniesize->setValue(12);
  QObject::connect( sboxliniesize, SIGNAL( valueChanged (int)), this, SLOT(setlFontSize(int)));
  labelliniesize = new QLabel(tr("Schriftgroesse Messinfo","Label"),gcolor);
  labelliniesize->setAlignment(Qt::AlignCenter);

  gdatei = new QGroupBox(tr("Messreihen"),woption);
  edateiname = new QLineEdit("default.csv",gdatei);
  labeldateiname = new QLabel(tr("Dateiname","Label"), gdatei);
  sphnr = new QSpinBox(gdatei);
  sphnr->setRange(1,1000);
  sphnr->setValue(1);
  labelhnr = new QLabel(tr("ObjektNr:","Label"), gdatei);
  checkaufzeichnung = new QCheckBox(tr("Aufzeichnen"), gdatei);
  QObject::connect( checkaufzeichnung, SIGNAL( stateChanged ( int )), this, SLOT(clickaufzeichnung(int)));
  
//#############################################################################
//Test Widget
//#############################################################################
  wtest = new QWidget();
  awidget->addTab(wtest, tr("HW Test","TabWindow"));
  
  grelais = new QGroupBox(tr("Relaisansteuerung"),wtest);
  bucplus = new QPushButton(grelais);
  bucplus->setText(tr("Kollektor+ Basis+"));
  QObject::connect( bucplus, SIGNAL( clicked()), this, SLOT(clickucplus()));
  bucminus = new QPushButton(grelais);
  bucminus->setText(tr("Kollektor- Basis+"));
  QObject::connect( bucminus, SIGNAL( clicked()), this, SLOT(clickucminus()));
  bubplus = new QPushButton(grelais);
  bubplus->setText(tr("Kollektor+ Basis-"));
  QObject::connect( bubplus, SIGNAL( clicked()), this, SLOT(clickubplus()));
  bubminus = new QPushButton(grelais);
  bubminus->setText(tr("Kollektor- Basis-"));
  QObject::connect( bubminus, SIGNAL( clicked()), this, SLOT(clickubminus()));
  b10uA = new QPushButton(grelais);
  b10uA->setText(tr("Basisstrom 10uA"));
  QObject::connect( b10uA, SIGNAL( clicked()), this, SLOT(clickb10uA()));
  b100uA = new QPushButton(grelais);
  b100uA->setText(tr("Basisstrom 100uA"));
  QObject::connect( b100uA, SIGNAL( clicked()), this, SLOT(clickb100uA()));
  b1mA = new QPushButton(grelais);
  b1mA->setText(tr("Basisstrom 1mA"));
  QObject::connect( b1mA, SIGNAL( clicked()), this, SLOT(clickb1mA()));
  
  gspannungen = new QGroupBox(tr("Spannung"),wtest);
  buc0v = new QPushButton(gspannungen);
  buc0v->setText(tr("Kollektor 0 Volt"));
  QObject::connect( buc0v, SIGNAL( clicked()), this, SLOT(clickuc0v()));
  buc10v = new QPushButton(gspannungen);
  buc10v->setText(tr("Kollektor 10 Volt"));
  QObject::connect( buc10v, SIGNAL( clicked()), this, SLOT(clickuc10v()));
  bub0v = new QPushButton(gspannungen);
  bub0v->setText(tr("Basis 0 Volt"));
  QObject::connect( bub0v, SIGNAL( clicked()), this, SLOT(clickub0v()));
  bub10v = new QPushButton(gspannungen);
  bub10v->setText(tr("Basis 10 Volt"));
  QObject::connect( bub10v, SIGNAL( clicked()), this, SLOT(clickub10v()));
  spucvar = new QSpinBox(gspannungen);
  QObject::connect(spucvar, SIGNAL(valueChanged(int)), this, SLOT(setucvar(int)));
  spucvar->setRange(0,10);
  spucvar->setSingleStep(1);
  lucvar = new QLabel(tr("Kollektor var."),gspannungen);
  lucvar->setAlignment(Qt::AlignCenter);
  spubvar = new QSpinBox(gspannungen);
  QObject::connect(spubvar, SIGNAL(valueChanged(int)), this, SLOT(setubvar(int)));
  spubvar->setRange(0,10);
  spubvar->setSingleStep(1);
  lubvar = new QLabel(tr("Basis variabel"),gspannungen);
  lubvar->setAlignment(Qt::AlignCenter);

  ganzeige = new QGroupBox(tr("Spannungsanzeige"),wtest);
  lademitter = new QLabel(tr("A/D Emitter: ","Label"), ganzeige);
  ladbasis = new QLabel(tr("A/D Basis: ","Label"), ganzeige);
  ladcollector = new QLabel(tr("A/D Kollektor: ","Label"), ganzeige);
  liemitter = new QLabel(tr("Ic/s: ","Label"), ganzeige);
  lubasis = new QLabel(tr("Ub/g: ","Label"), ganzeige);
  lucollector = new QLabel(tr("Uc/d: ","Label"), ganzeige);;
  spadcoffset = new QSpinBox(ganzeige);
  QObject::connect(spadcoffset, SIGNAL(valueChanged(int)), this, SLOT(setadcoffset(int)));
  spadcoffset->setRange(-30,0);
  spadcoffset->setSingleStep(1);
  ladcoffset = new QLabel(tr("ADC-Offset"),ganzeige);
  ladcoffset->setAlignment(Qt::AlignCenter);

  gstrommax = new QGroupBox(tr("Einstellung Imax"),wtest);
  spstrommax = new QSpinBox(gstrommax);
  spstrommax->setRange(5,500);
  spstrommax->setSingleStep(10);
  spstrommax->setValue(200);
  labelspstrommax = new QLabel(tr("(mA)","Label"), gstrommax);
  lstrommax = new QLabel(tr("Ubmax: 0,0 Volt","Label"), gstrommax);
  luc = new QLabel(tr("Uc: 0,0 Volt","Label"), gstrommax);
  bstrommax = new QPushButton(gstrommax);
  bstrommax->setText(tr("Ermitteln"));
  QObject::connect( bstrommax, SIGNAL( clicked()), this, SLOT(clickbstrommax()));
  
  gdatest = new QGroupBox(tr("D/A Dauertest"),wtest);
  bdastart = new QPushButton(gdatest);
  bdastart->setText(tr("Start"));
  QObject::connect( bdastart, SIGNAL( clicked()), this, SLOT(clickbdastart()));
  
  ucpol = 1.0;
  ubpol = 1.0;
//----------------------------------------------------------------  
  createActions();
  createMenus();
  createToolBars();
  createStatusBar();
  
  awidget->setEnabled(true);
  kurvennummer = 0;
  
  repainttimer = new QTimer(this);
  QObject::connect(repainttimer, SIGNAL(timeout()), this, SLOT(actrepainttimer() ));
  openttytimer = new QTimer(this);
  QObject::connect(openttytimer, SIGNAL(timeout()), this, SLOT(fopentty() ));
  #ifdef Q_WS_WIN
    int i;
    QString qs;
    for(i=1;i<=20;i++){
      qs.sprintf("COM%i",i);
      listtty.append(qs);
    }
  #else
    listtty.append("/dev/ttyS0");
    listtty.append("/dev/ttyS1");
    listtty.append("/dev/ttyS2");
    listtty.append("/dev/ttyS3");
    listtty.append("/dev/ttyUSB0");
    listtty.append("/dev/ttyUSB1");
    listtty.append("/dev/ttyUSB2");
    listtty.append("/dev/ttyUSB3");
  #endif  
  idtty = 0; //mit Index 0 beginnen
  bvantwort = false;
  bksgefunden = false;
  readSettings();
  grafiksetzen();
  awidget->setEnabled(false);
  repainttimer->start(1);
  bwstart = false;
  kurvenende = false;
}

void kschreiber::rs232einstellen()
{
  bool ok;
  
//QString getText ( QWidget * parent, const QString & title, const QString & label, QLineEdit::EchoMode mode = QLineEdit::Normal, const QString & text = QString(), bool * ok = 0, Qt::WindowFlags f = 0 )
  srs232 = QInputDialog::getText(this, tr("RS232","in InputBox"), tr("Einstellen der seriellen Schnittstelle"),
                                 QLineEdit::Normal,
                                 srs232, &ok);
  if(ok){
    picmodem->closetty();
    rs232isopen = picmodem->opentty(srs232);
    if(!rs232isopen){
      QMessageBox::about(this, tr("RS232 einstellen"),tr("Die RS232 ist nicht offen!"));
    }
  }
}

void kschreiber::setAText(QString stty){
  QString qs;
  qs = tr("Kennlinien-Schreiber");
  QString fwversion = tr("HW nicht gefunden");
  if(ksettings.klsfwversion >= 100)fwversion.sprintf("FW:%i.%02i",ksettings.klsfwversion/100, ksettings.klsfwversion -((ksettings.klsfwversion/100)*100));
  QString astty = qs + " - " + ksettings.version + " - " + stty + " - " + fwversion;
  setWindowTitle(astty);
}

void kschreiber::closeEvent(QCloseEvent *event)
{
  cfgspeichern();
  writeSettings();
  if(checkaufzeichnung->isChecked()){
    clickaufzeichnung(Qt::Unchecked); 
  }
  event->accept();
}

void kschreiber::about()
{
  QSettings settings("AFU", "kschreiber");
  QDir imgpath;
  imgpath.setPath(settings.value("imgpath", QDir::homePath()).toString());
  QString qs=tr("<p><b>Bilder:</b> %1</p>").arg(imgpath.absolutePath());
  QString qs1;
  qs1.sprintf("<p><b>PIC-Firmware:</b> %i.%02i</p",ksettings.klsfwversion/100, ksettings.klsfwversion -((ksettings.klsfwversion/100)*100));
#ifdef Q_WS_WIN
  QMessageBox::about(this, "PC Software-KLSwin", infotext+qs1+qs);
#else
  QMessageBox::about(this, "PC Software-KLSlin", infotext+qs1+qs);
#endif
}

void kschreiber::createActions()
{
  labelAct = new QAction(QIcon(":/label.png"), tr("Beschreibung zum Bild","Menu"), this);
  labelAct->setStatusTip(tr("Beschreibung zum Bild eintragen", "Statustip"));
  connect(labelAct, SIGNAL(triggered()), this, SLOT(labelspeichern()));

  savegrafikAct = new QAction(QIcon(":/savebild.png"), tr("Speichern als Bild","Menu"), this);
  savegrafikAct->setStatusTip(tr("Speichern der Grafik als Bild-Datei", "Statustip"));
  connect(savegrafikAct, SIGNAL(triggered()), this, SLOT(bildspeichern()));

  exitAct = new QAction(QIcon(":/beenden.xpm"),tr("&Beenden"), this);
  exitAct->setStatusTip(tr("Programm beenden"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

  rs232Act = new QAction(tr("&RS232 einstellen"), this);
  rs232Act->setStatusTip(tr("Serielle Schnittstelle einstellen"));
  connect(rs232Act, SIGNAL(triggered()), this, SLOT(rs232einstellen()));

  aboutAct = new QAction(QIcon(":/info.png"),tr("&Ueber"), this);
  aboutAct->setStatusTip(tr("Ueber die Applikation"));
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

  aboutQtAct = new QAction(tr("Ueber &Qt"), this);
  aboutQtAct->setStatusTip("Show the Qt library's About box");
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
  
  kennlausgangAct = new QAction(QIcon(":/kaus.xpm"),tr("&Ausgangskennlinie"), this);
  kennlausgangAct->setStatusTip(tr("Ausgangskennlinie darstellen"));
  connect(kennlausgangAct, SIGNAL(triggered()), this, SLOT(kennlausgang()));

  kennleingangAct = new QAction(QIcon(":/kein.xpm"),tr("&Eingangskennlinie"), this);
  kennleingangAct->setStatusTip(tr("Eingangskennlinie darstellen"));
  connect(kennleingangAct, SIGNAL(triggered()), this, SLOT(kennleingang()));

  npnAct = new QAction(QIcon(":/knpn.xpm"),tr("&NPN Transistor"), this);
  npnAct->setStatusTip(tr("Einstellung auf NPN Transistoren"));
  connect(npnAct, SIGNAL(triggered()), this, SLOT(setnpn()));
  
  pnpAct = new QAction(QIcon(":/kpnp.xpm"),tr("&PNP Transistor"), this);
  pnpAct->setStatusTip(tr("Einstellung auf PNP Transistoren"));
  connect(pnpAct, SIGNAL(triggered()), this, SLOT(setpnp()));
  
  nmosfetAct = new QAction(QIcon(":/knmos.xpm"),tr("N &Mosfet"), this);
  nmosfetAct->setStatusTip(tr("Einstellung auf N Mosfet"));
  connect(nmosfetAct, SIGNAL(triggered()), this, SLOT(setnmosfet()));
  
  pmosfetAct = new QAction(QIcon(":/kpmos.xpm"),tr("P M&osfet"), this);
  pmosfetAct->setStatusTip(tr("Einstellung auf P Mosfet"));
  connect(pmosfetAct, SIGNAL(triggered()), this, SLOT(setpmosfet()));
  
  njfetAct = new QAction(QIcon(":/knfet.xpm"),tr("N &JFet"), this);
  njfetAct->setStatusTip(tr("Einstellung auf N JFet"));
  connect(njfetAct, SIGNAL(triggered()), this, SLOT(setnjfet()));
  
  pjfetAct = new QAction(QIcon(":/kpfet.xpm"),tr("P J&Fet"), this);
  pjfetAct->setStatusTip(tr("Einstellung auf P JFet"));
  connect(pjfetAct, SIGNAL(triggered()), this, SLOT(setpjfet()));

  diodeAct = new QAction(QIcon(":/kdiode.xpm"),tr("&Diode"), this);
  diodeAct->setStatusTip(tr("Einstellung auf Dioden"));
  connect(diodeAct, SIGNAL(triggered()), this, SLOT(setdiode()));

  savecsvAct = new QAction(QIcon(":/filesavecsv.xpm"),tr("&Save Kurve als CSV Datei"), this);
  savecsvAct->setStatusTip(tr("Kurve speichern als CSV Datei"));
  connect(savecsvAct, SIGNAL(triggered()), this, SLOT(clicksavecsv()));
}

void kschreiber::createMenus()
{
  fileMenu = menuBar()->addMenu(tr("&Datei"));
  fileMenu->addAction(savecsvAct);
  fileMenu->addAction(savegrafikAct);
  fileMenu->addAction(labelAct);
  fileMenu->addSeparator();
  fileMenu->addAction(exitAct);
  
  optionMenu = menuBar()->addMenu(tr("&Einstellungen"));
  optionMenu->addAction(rs232Act);

  kschreiberMenu = menuBar()->addMenu(tr("&Kennlinien"));
  kschreiberMenu->addAction(kennlausgangAct);
  kschreiberMenu->addAction(kennleingangAct);

  transistortypMenu = menuBar()->addMenu(tr("&Transistortyp"));
  transistortypMenu->addAction(npnAct);
  transistortypMenu->addAction(pnpAct);
  transistortypMenu->addAction(nmosfetAct);
  transistortypMenu->addAction(pmosfetAct);
  transistortypMenu->addAction(njfetAct);
  transistortypMenu->addAction(pjfetAct);
  transistortypMenu->addAction(diodeAct);
  
  menuBar()->addSeparator();

  helpMenu = menuBar()->addMenu(tr("&Hilfe"));
  helpMenu->addAction(aboutAct);
  helpMenu->addAction(aboutQtAct);
}

void kschreiber::createToolBars()
{
  fileToolBar = addToolBar(tr("Datei"));
  fileToolBar->addAction(exitAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(kennleingangAct);
  fileToolBar->addAction(kennlausgangAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(npnAct);
  fileToolBar->addAction(pnpAct);
  fileToolBar->addAction(nmosfetAct);
  fileToolBar->addAction(pmosfetAct);
  fileToolBar->addAction(njfetAct);
  fileToolBar->addAction(pjfetAct);
  fileToolBar->addAction(diodeAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(savecsvAct);
  fileToolBar->addAction(savegrafikAct);
  fileToolBar->addAction(labelAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(aboutAct);
}

void kschreiber::kennlausgang(){
  getUI();
  if(kurventyp == eeingang){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  kurventyp = eausgang;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::kennleingang(){
  getUI();
  if(kurventyp == eausgang){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  kurventyp = eeingang;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setnpn(){
  getUI();
  if(ksettings.htyp != enpn){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = enpn;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setpnp(){
  getUI();
  if(ksettings.htyp != epnp){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = epnp;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setnmosfet(){
  getUI();
  if(ksettings.htyp != enmosfet){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = enmosfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setpmosfet(){
  getUI();
  if(ksettings.htyp != epmosfet){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = epmosfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setnjfet(){
  getUI();
  if(ksettings.htyp != enjfet){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = enjfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setpjfet(){
  getUI();
  if(ksettings.htyp != epjfet){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = epjfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setdiode(){
  getUI();
  if(ksettings.htyp != ediode){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  ksettings.htyp = ediode;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setKurvenSchreiber(){
  int p=0;
  QString qs;
  
  switch(kurventyp){
    case eeingang:
      switch(ksettings.htyp){
        case enpn:
          gibasis->show();
          gibasis->setEnabled(true);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Ibe (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.xmasseinheit="uA";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Uce (V)");
          ksettings.setueberschrift(tr("NPN-Eingangskennlinienfeld Ic = f (Ibe)"));
          gkurven->setTitle(tr("Kurve-NPN-Eingang"));
          labelui1->setText("Uce (V):");
          labelui2->setText("Icmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epnp:
          gibasis->show();
          gibasis->setEnabled(true);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Ibe (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.xmasseinheit="uA";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Uce (V)");
          ksettings.setueberschrift(tr("PNP-Eingangskennlinienfeld Ic = f (Ibe)"));
          gkurven->setTitle(tr("Kurven-PNP-Eingang"));
          labelui1->setText("Uce (V):");
          labelui2->setText("Icmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("N Mosfet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-N Mosfet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("P Mosfet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-P Mosfet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enjfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(-10.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("N JFet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-N JFet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = -1.0;
          p = 2;
        break;
        case epjfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(-10.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("P JFet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-P JFet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = 1.0;
          p = 1;
        break;
        case ediode:
          gibasis->show();
          gibasis->setEnabled(true);
          gibasis->setTitle(tr("I-Diode"));
          ksettings.setxeinheit("I-Diode (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Uac (V)");
          ksettings.xmasseinheit="uA";
          ksettings.ymasseinheit="V";
          ksettings.setzeinheit("");
          ksettings.setueberschrift(tr("Diodenkennlinienfeld Uac = f (I-Diode)"));
          gkurven->setTitle(tr("Kurve-Diode"));
          labelui1->setText("Uac (V):");
          labelui2->setText("");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
      }
      break;
    case eausgang:
      switch(ksettings.htyp){
        case enpn:
          gibasis->show();
          gibasis->setEnabled(true);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Uce (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Ibe (uA)");
          ksettings.setueberschrift(tr("NPN-Ausgangskennlinienfeld Ic = f (Uce)"));
          gkurven->setTitle(tr("Kurven-NPN-Ausgang"));
          labelui1->setText("Ib (uA):");
          labelui2->setText("Icmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epnp:
          gibasis->show();
          gibasis->setEnabled(true);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Uce (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Ibe (uA)");
          ksettings.setueberschrift(tr("PNP-Ausgangskennlinienfeld Ic = f (Uce)"));
          gkurven->setTitle(tr("Kurven-PNP-Ausgang"));
          labelui1->setText("Ib (uA):");
          labelui2->setText("Icmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("N Mosfet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-N Mosfet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("P Mosfet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-P Mosfet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enjfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("N JFet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-N JFet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = -1.0;
          p = 2;
        break;
        case epjfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.xmasseinheit="V";
          ksettings.ymasseinheit="mA";
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("P JFet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-P JFet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = 1.0;
          p = 1;
        break;
        case ediode:
          gibasis->show();
          gibasis->setEnabled(true);
          gibasis->setTitle(tr("I-Diode"));
          ksettings.setxeinheit("I-Diode (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Uac (V)");
          ksettings.xmasseinheit="uA";
          ksettings.ymasseinheit="V";
          ksettings.setzeinheit("");
          ksettings.setueberschrift(tr("Diodenkennlinienfeld Uac = f (I-Diode)"));
          gkurven->setTitle(tr("Kurve-Diode"));
          labelui1->setText("Uac (V):");
          labelui2->setText("");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
      }
    break;
  }
  textedit->clear();
  int index = boxui2->currentIndex();
  aendernboxui2(index);
  grafik->setPlotSettings(ksettings);
  boxui1fuellen();
  boxui2fuellen();
  //qDebug(qPrintable(srs232));
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    qs.sprintf("@p%1i",p);
    //qDebug(qPrintable(qs));
    picmodem->writeLine(qs);
  }else{
    //warmstart();
  }
  grafiksetzen();
}

void kschreiber::createStatusBar()
{
  statusBar()->showMessage(tr("Ready"));
}

void kschreiber::readSettings()
{
  int i;
  QSettings settings("AFU", "kschreiber");
  QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
  QSize size = settings.value("size", QSize(400, 400)).toSize();
  move(pos);
  resize(size);
  #ifdef Q_WS_WIN
  srs232 = settings.value("rs232", QString("COM1")).toString();
  #else
  srs232 = settings.value("rs232", QString("/dev/ttyS0")).toString();
  #endif
  
  picmodem->closetty();
  if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    defaultlesentty();
    picmodem->writeLine("@v");
    bvantwort = true;
  }else{
    //warmstart();
  }
  openttytimer->start(200); //des Geraetes
  i = settings.value("kurventyp", int(eeingang)).toInt();
  kurventyp = (ekurventyp)i;
  i = settings.value("halbleitertyp", int(enmosfet)).toInt();
  ksettings.htyp = (ehalbleitertyp)i;
  cfgladen();
  setKurvenSchreiber();
  setUI();
}

void kschreiber::writeSettings()
{
  QSettings settings("AFU", "kschreiber");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.setValue("rs232", srs232);
  settings.setValue("kurventyp", int(kurventyp));
  settings.setValue("halbleitertyp", int(ksettings.htyp));
  getUI();
  cfgspeichern();
}


kschreiber::~kschreiber()
{
}

void kschreiber::defaultlesentty(){
  unsigned char abuffer[4098];

  if (rs232isopen){
    picmodem->readttybuffer(abuffer, 4098);
  }
}

void kschreiber::clickucplus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p0");
    ucpol = 1.0;
    ubpol = 1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickucminus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p1");
    ucpol = -1.0;
    ubpol = 1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickubplus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p2");
    ucpol = 1.0;
    ubpol = -1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickubminus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p3");
    ucpol = -1.0;
    ubpol = -1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickb10uA(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@a0");
  }else{
    warmstart();
  }
}

void kschreiber::clickb100uA(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@a1");
  }else{
    warmstart();
  }
}

void kschreiber::clickb1mA(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@a2");
  }else{
    warmstart();
  }
}

void kschreiber::clickuc0v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@c0000");
  }else{
    warmstart();
  }
}

void kschreiber::clickuc10v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@c1023");
  }else{
    warmstart();
  }
}

void kschreiber::clickub0v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@b0000");
  }else{
    warmstart();
  }
}

void kschreiber::clickub10v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@b1023");
  }else{
    warmstart();
  }
}

void kschreiber::setucvar(int u){
  QString qs;
  const double faktor = 1023.0 / 10.0;
  int adc;
  if(rs232isopen){
    adc = int(round(double(u) * faktor));
    qs.sprintf("@c%04i",adc);
    //qDebug()<<qs;
    picmodem->writeLine(qs);
  }else{
    warmstart();
  }
}


void kschreiber::setubvar(int u){
  QString qs;
  const double faktor = 1023.0 / 10.0;
  int adc;
  if(rs232isopen){
    adc = int(round(double(u) * faktor));
    qs.sprintf("@b%04i",adc);
    //qDebug()<<qs;
    picmodem->writeLine(qs);
  }else{
    warmstart();
  }
}

void kschreiber::clicksavecsv(){
  QFile f;
  QString qs, qs1, qs2, qs3, sdatei;
  unsigned char c1, c2;
  int messd, i, j, k, l=0;
  const double uconst = 10.0/kadcmax;
  const double uconstjfet = 10.0/kadcmax;
  const double iconst = 500.0/kadcmax;
  double wert1=0.0, wert2=0.0;
  int readmax=0;

  //Festlegen der Datenmenge die maximal zurueck kommt
  switch(ksettings.htyp){
  case enpn:
  case epnp:
  case enmosfet:
  case epmosfet:
  case ediode:
    readmax = 2050;
    break;
  case enjfet:
  case epjfet:
    readmax = 4098;
    break;
  }
  if(kurvenende){
    qs2 = boxui1->currentText();
    qs3 = boxui2->currentText();
    switch(kurventyp){
    case eeingang:
      switch(ksettings.htyp){
      case enpn:
        qs = "# NPN Eingangskennlinie Ic[mA]=f(Ib) Uc[V]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Ib[uA];Ic[mA]";
        break;
      case epnp:
        qs = "# PNP Eingangskennlinie Ic[mA]=f(Ib) Uc[V]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Ib[uA];Ic[mA]";
        break;
      case enmosfet:
        qs = "# N-Mosfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case epmosfet:
        qs = "# P-Mosfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case enjfet:
        qs = "# N-Jfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case epjfet:
        qs = "# P-Jfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case ediode:
        qs = "# Diode Testkennlinie Uac[V]=f(Diode I)";
        qs1 = "Idiode[uA];Uac[V]";
        break;
      }
      break;
    case eausgang:
      switch(ksettings.htyp){
      case enpn:
        qs = "# NPN Ausgangskennlinie Ic[mA]=f(Uce) Ib[uA]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Uce[V];Ic[mA]";
        break;
      case epnp:
        qs = "# PNP Ausgangskennlinie Ic[mA]=f(Uce) Ib[uA]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Uce[V];Ic[mA]";
        break;
      case enmosfet:
        qs = "# N-Mosfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case epmosfet:
        qs = "# P-Mosfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case enjfet:
        qs = "# N-Jfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case epjfet:
        qs = "# P-Jfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case ediode:
        qs = "# Diode Testkennlinie Uac[V]=f(Diode I)";
        qs1 = "Idiode[uA];Uac[V]";
        break;
      }
      break;
    }

    sdatei = QFileDialog::getSaveFileName(
          this
          ,tr("Kurve als CSV Speichern","FileDialog")
          ,"kurve.csv"
          ,tr("CSV Datei (*.csv *.csv);;""Alle Dateien (*.*)","FileDialog")
          );
    if(!sdatei.isEmpty()){
      f.setFileName(sdatei);
      if(f.open(QIODevice::WriteOnly)){
        QTextStream ds(&f);
        ds << qs << "\r\n";
        ds << qs1 << "\r\n";
        for(i=0; i<readmax; i+=2){
          c1 = rxbuffer[i];
          c2 = rxbuffer[i+1];
          messd = adc2int(c1,c2);
          //messd = double(c1 + c2 * 256 + kadcoffset) ;
          if(ksettings.klsfwversion>=200)messd=messd/4.0;
          //qDebug("messd:%f",messd);
          if(messd == double(0x8000)){
            kurvenende = true;
            l = i - 2;
          }
          if(i == readmax){
            kurvenende = true;
            l = i - 2;
          }
        }
        k=0;
        //qDebug("readmax %i",readmax);
        //qDebug("wert l %i",l);
        for(j=0; j<l; j+=2){
          c1 = rxbuffer[j];
          c2 = rxbuffer[j+1];
          messd = adc2int(c1,c2);
          //messd = double(c1 + c2 * 256 + kadcoffset);
          if(ksettings.klsfwversion>=200)messd=messd/4.0;
          //qDebug("messd %f",messd);
          switch(kurventyp){
          case eeingang:
            switch(ksettings.htyp){
            case enpn:
            case epnp:
              wert1= double(k)*getIbconst();
              wert2= double(messd)*iconst;
              break;
            case ediode:
              wert1= double(k)*getIbconst();
              wert2= double(messd)*uconst;
              //messkurve.append(QPointF(double(k)*getIbconst(),double(messd)*iconst));
              break;
            case enmosfet:
            case epmosfet:
              wert1= double(k)*uconst;
              wert2= double(messd)*iconst;
              //messkurve.append(QPointF(double(k)*uconst,double(messd)*iconst));
              break;
            case enjfet:
            case epjfet:
              wert1= (double(k)*uconstjfet)-10.0;
              wert2= double(messd)*iconst;
              //messkurve.append(QPointF((double(k)*uconstjfet)-10.0,double(messd)*iconst));
              break;
            }
            break;
          case eausgang:
            switch(ksettings.htyp){
            case enpn:
            case epnp:
            case enmosfet:
            case epmosfet:
            case enjfet:
            case epjfet:
              wert1= double(k)*uconst;
              wert2= double(messd)*iconst;
              //messkurve.append(QPointF(double(k)*uconst,double(messd)*iconst));
              break;
            case ediode:
              wert1= double(k)*getIbconst();
              wert2= double(messd)*uconst;
              //messkurve.append(QPointF(double(k)*getIbconst(),double(messd)*uconst));
              break;
            }

            break;
          }
          qs.sprintf("%2.4f;%2.4f",wert1,wert2);
          qs.replace('.',',');
          ds << qs << "\r\n";
          k++;
        }
        f.close();
      }
    }
  }else{
    QMessageBox::about(this, tr("CSV Datei speichern"),tr("Es wurde noch keine Kurve erzeugt!"));
  }
}

void kschreiber::settabwidget(int a){
  switch(a){
    case 0:
      testtimer->stop();
      imaxtimer->stop();
      defaultlesentty();
    break;
    case 1:
      defaultlesentty();
      testtimer->stop();
      akurventimer->stop();
      ekurventimer->stop();
    break;
    case 2:
      defaultlesentty();
      adindex = 0;
      if(rs232isopen){
        picmodem->writeLine("@m0");
      }else{
        warmstart();
      }
      akurventimer->stop();
      ekurventimer->stop();
      testtimer->start(100);
      rxindex = 0;
    break;
  }
}

void kschreiber::readbfm(){
  unsigned char abuffer[2];
  unsigned char c1, c2;
  int bad;
  QString qs;
  double ieconst = (1.0/kadcmax)*500.0;
  double uconst = (1.0/kadcmax)*10.0;
  int gelesen;
  
  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, 2);
    if(gelesen > 0){
      for(int i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
      testtimer->start(100);
    }
    if(rxindex >= 2){
      syncounter = 0;
      rxindex = 0;
      c1 = rxbuffer[0];
      c2 = rxbuffer[1];
      bad = adc2int(c1,c2);
      //bad = int(c1 + c2 * 256 + kadcoffset);
      switch(adindex){
      case 0:
        qs.sprintf(" %i",bad);
        qs = tr("A/D Basis:") + qs;
        ladbasis->setText(qs);
        if(ksettings.klsfwversion<200){
          qs.sprintf("Ub: %02.2f V", uconst * double(bad) * ubpol);
        }else{
          //qs.sprintf("Ub: %02.2f V", uconst * double(bad) * ubpol /4.0);
          qs.sprintf("Ub: %02.2f V", uconst * double(bad) * ubpol);
        }
        lubasis->setText(qs);
        break;
      case 1:
        qs.sprintf(" %i",bad);
        qs = tr("A/D Emitter:") + qs;
        lademitter->setText(qs);
        if(ksettings.klsfwversion<200){
          qs.sprintf("Ic: %03.1f mA", ieconst * double(bad) * ucpol);
        }else{
          //qs.sprintf("Ic: %03.1f mA", ieconst * double(bad) * ucpol /4.0);
          qs.sprintf("Ic: %03.1f mA", ieconst * double(bad) * ucpol);
        }
        liemitter->setText(qs);
        //qDebug("bad:%i",bad);
        break;
      case 2:
        qs.sprintf(" %i",bad);
        qs = tr("A/D Kollektor:") + qs;
        ladcollector->setText(qs);
        if(ksettings.klsfwversion<200){
          qs.sprintf("Uc: %02.2f V", uconst * double(bad) * ucpol);
        }else{
          //qs.sprintf("Uc: %02.2f V", uconst * double(bad) * ucpol / 4.0);
          qs.sprintf("Uc: %02.2f V", uconst * double(bad) * ucpol);
        }
        lucollector->setText(qs);
        break;
      }
      adindex++;
      if(adindex==3)adindex=0;
      picmodem->writeChar('@');
      switch(adindex){
      case 0:
        picmodem->writeLine("m0");
        break;
      case 1:
        picmodem->writeLine("m1");
        break;
      case 2:
        picmodem->writeLine("m2");
        break;
      }
      defaultlesentty();
      testtimer->start(100);
    }else{
      adindex=0;
      picmodem->writeLine("@m0");
      syncounter++;
      if(syncounter == 5){
        warmstart();
        awidget->setCurrentIndex(0);
      }
    }
  }else{
    warmstart();
  }
}

void kschreiber::clickbstrommax(){
  int intimax;
  const double ieconst = (1.0/kadcmax)*500.0;
  QString qs;
  
  if(rs232isopen){
    picmodem->writeLine("@c1023");
  }else{
    warmstart();
  }
  double imax = spstrommax->value();
  intimax = abs(imax / ieconst);
  //qDebug("intimax %i",intimax);
  qs.sprintf("@t%04i",intimax);
  if(rs232isopen){
    picmodem->writeLine(qs);
  }else{
    warmstart();
  }
  defaultlesentty();
  testtimer->stop();
  rxindex = 0;
  imaxtimer->start(500);
}

void kschreiber::readimax(){
  unsigned char abuffer[4];
  unsigned char c1, c2;
  int bad1, bad2;
  double ubasis;
  QString qs;
  double uconst = 10.0/1023;
  double adcconst = 10.0/kadcmax;
  int gelesen;

  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, 4);
    if(gelesen > 0){
      for(int i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
    }
    if(rxindex >= 4){
      rxindex = 0;
      c1 = rxbuffer[0];
      c2 = rxbuffer[1];
      bad1 = adc2int(c1,c2);
      c1 = rxbuffer[2];
      c2 = rxbuffer[3];
      bad2 = adc2int(c1,c2);
      ubasis = bad1 * uconst;
      qs.sprintf("Ubmax: %2.3f Volt",ubasis);
      if(ubasis > 10.0)qs=tr("Eingaenge offen!!!");
      lstrommax->setText(qs);
      ubasis = bad2 * adcconst;
      qs.sprintf("Uc: %2.3f Volt",ubasis);
      luc->setText(qs);
      defaultlesentty();
      picmodem->writeLine("@c0000");
      testtimer->start(100);
    }
  }else{
    warmstart();
  }
}

void kschreiber::resizeEvent( QResizeEvent * ){
  //nach Groessenaenderung grafische Elemente anpassen
  resize(this->rect().size());
  grafiksetzen();
  //qDebug("breite %i hoehe %i", gsize.width(), gsize.height());
}

void kschreiber::repaintEvent( QPaintEvent * ){
  //nach Groessenaenderung grafische Elemente anpassen
  //qDebug("breite %i hoehe %i", width(), height());
  grafiksetzen();
  QWidget::repaint();
}

void kschreiber::grafiksetzen(){
//  QSize gsize = wkschreiber->size();
//  grafik->resize(gsize);
//  grafik->setParameterPixmap();

  int w = wkschreiber->width();
  int h = wkschreiber->height();
  int gl = w-200;
  int a = 20;
  int b = 20;
  int f = 0;
  
  grafik->setGeometry(0,0,gl,h);
  
  gkurven->setGeometry(gl+10,0,180,170);
  bkurvestart->setGeometry(10, b+f*a, 150, 20);f++;
  bclearcurven->setGeometry(10, b+f*a, 150, 20);f++;
  bsethintergrund->setGeometry(10, b+f*a, 150, 20);f++;
  labelui1->setGeometry(10, b+f*a, 80, 20); 
  boxui1->setGeometry(90, b+f*a, 70, 20);f++;
  labelui2->setGeometry(10, b+f*a, 80, 20); 
  boxui2->setGeometry(90, b+f*a, 70, 20);f++;
  checkclrcurve->setGeometry(10, b+f*a, 150, 20);f++;

  gibasis->setGeometry(gl+10,175,180,90);
  i10ua->setGeometry(10,20,160,20);
  i100ua->setGeometry(10,40,160,20);
  i1ma->setGeometry(10,60,160,20);
  
  switch(ksettings.htyp){
    case enpn:
    case epnp:
    case ediode:
      textedit->setGeometry(gl+10,275,180,h-280-10);
    break;
    default:
      textedit->setGeometry(gl+10,180,180,h-180-10);
    break;
  }
  
  grelais->setGeometry(10,10,180,190);
  bucplus->setGeometry(10,30,150,20);
  bucminus->setGeometry(10,50,150,20);
  bubplus->setGeometry(10,70,150,20);
  bubminus->setGeometry(10,90,150,20);
  b10uA->setGeometry(10,110,150,20);
  b100uA->setGeometry(10,130,150,20);
  b1mA->setGeometry(10,150,150,20);
  
  gspannungen->setGeometry(10,210,180,190);
  buc0v->setGeometry(10,30,150,20);
  buc10v->setGeometry(10,50,150,20);
  bub0v->setGeometry(10,70,150,20);
  bub10v->setGeometry(10,90,150,20);
  spucvar->setGeometry(10,120,50,25);
  spubvar->setGeometry(10,150,50,25);
  lucvar->setGeometry(70,122,100,20);
  lubvar->setGeometry(70,152,100,20);

  ganzeige->setGeometry(220,10,150,190);
  lademitter->setGeometry(10,30,120,20);
  ladbasis->setGeometry(10,50,120,20);
  ladcollector->setGeometry(10,70,120,20);
  liemitter->setGeometry(10,100,120,20);
  lubasis->setGeometry(10,120,120,20);
  lucollector->setGeometry(10,140,120,20);
  spadcoffset->setGeometry(10,160,50,25);
  ladcoffset->setGeometry(70,162,70,20);

  gstrommax->setGeometry(220,210,150,130);
  spstrommax->setGeometry(10,30,70,20);
  labelspstrommax->setGeometry(90,30,60,20);
  lstrommax->setGeometry(10,50,130,20);
  luc->setGeometry(10,70,130,20);
  bstrommax->setGeometry(10,90,100,20);
  
  gdatest->setGeometry(220,350,150,60);
  bdastart->setGeometry(10,30,100,20);
  
  gcolor->setGeometry(10,10,250,220);
  bcolorhintergrund->setGeometry(10,20,150,20);
  bcolorschrift->setGeometry(10,40,150,20);
  bcolorgitter->setGeometry(10,60,150,20);
  bcolorueberschrift->setGeometry(10,80,150,20);
  bcolorlinie->setGeometry(10,100,150,20);
  bcolorhlinie->setGeometry(10,120,150,20);
  sboxliniefsize->setGeometry(10,140,50,30);
  labelliniefsize->setGeometry(60,140,180,30);
  sboxliniesize->setGeometry(10,170,50,30);
  labelliniesize->setGeometry(60,170,180,30);

  gdatei->setGeometry(270,10,250,100);
  edateiname->setGeometry(100,20,140,20);
  labeldateiname->setGeometry(10,20,80,20);
  sphnr->setGeometry(70,50,60,20);
  labelhnr->setGeometry(10,50,40,20);
  checkaufzeichnung->setGeometry(140,50,100,20);
  
//  wkschreiber->resize(size());
//  grafik->setPlotSettings();
}
void kschreiber::clickkurvenstart(){
  const double uconsty = 10.0/kadcmax;
  const double uconstx = 10.0/1023.0;
  const double iconsty = 500.0/kadcmax;
  //const double iconstx = 500.0/1023.0;
  double ub;
  double uc;
  double icmax;
  QString qs;
  
  //Empfangspuffer loeschen
  for(int i=0; i<4098; i++) rxbuffer[i] = char(0);
  messkurve.clear();
  if(checkclrcurve->isChecked())clickbclearcurven();
  switch(kurventyp){
  case eausgang:
    switch(ksettings.htyp){
    case enpn:
    case epnp:
      if(i10ua->isChecked())qs="@a0";
      if(i100ua->isChecked())qs="@a1";
      if(i1ma->isChecked())qs="@a2";
      //Ib-Bereich umschalten
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //qDebug(qPrintable(qs));
      //Wert im Bereich einstellen
      ub = getui1();
      ub = ub  /  getIbconst();
      qs.sprintf("@b%04.0f",ub);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //max Ic feststellen
      icmax = getui2() / iconsty;
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@k%04.0f",icmax);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //Timer starten zum lesen der Daten
      akurventimer->start(100);
      idlezaehler = 0;
      //RS232 Empfangsbuffer leeren
      defaultlesentty();
      rxindex = 0;
      break;
    case epmosfet:
    case enmosfet:
      //Immer Ib max 1mA einstellen
      if(rs232isopen){
        picmodem->writeLine("@a2");
      }else{
        warmstart();
      }
      //Box ui1 auslesen
      ub = getui1() / uconstx;
      qs.sprintf("@b%04.0f",ub);
      //qDebug(qPrintable(qs));
      //Gatespannung einstellen
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //Maximale Strom
      //Box ui2 auslesen
      icmax = getui2() / iconsty;
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@k%04.0f",icmax);
      //qDebug(qPrintable(qs));
      //Id max einstellen
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      akurventimer->start(100);
      idlezaehler = 0;
      defaultlesentty();
      rxindex = 0;
      break;
    case enjfet:
      //Immer Ib max 1mA einstellen
      if(rs232isopen){
        picmodem->writeLine("@a2");
      }else{
        warmstart();
      }
      //Box ui1 auslesen
      ub = getui1() / uconsty;
      //Polaritaet setzen
      if(ub <= 0.0)qs = "@p2";
      if(ub > 0.0)qs = "@p0";
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      ub = abs(getui1() / uconstx);
      qs.sprintf("@b%04.0f",ub);
      //qDebug(qPrintable(qs));
      //Gatespannung einstellen
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //Box ui2 auslesen
      icmax = getui2() / iconsty;
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@k%04.0f",icmax);
      //qDebug(qPrintable(qs));
      //Id max einstellen
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      akurventimer->start(100);
      idlezaehler = 0;
      defaultlesentty();
      rxindex = 0;
      break;
    case epjfet:
      //Immer Ib max 1mA einstellen
      if(rs232isopen){
        picmodem->writeLine("@a2");
      }else{
        warmstart();
      }
      //Box ui1 auslesen
      ub = getui1() / uconsty;
      if(ub <= 0.0)qs = "@p3";
      if(ub > 0.0)qs = "@p1";
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      ub = abs(getui1() / uconstx);
      qs.sprintf("@b%04.0f",ub);
      //qDebug(qPrintable(qs));
      //Gatespannung einstellen
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //Box ui2 auslesen
      icmax = getui2() / iconsty;
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@k%04.0f",icmax);
      //qDebug(qPrintable(qs));
      //Id max einstellen
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      akurventimer->start(100);
      idlezaehler = 0;
      defaultlesentty();
      rxindex = 0;
      break;
    case ediode:
      if(i10ua->isChecked())qs="@a0";
      if(i100ua->isChecked())qs="@a1";
      if(i1ma->isChecked())qs="@a2";
      //Ib-Bereich umschalten
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine("@g");
      }else{
        warmstart();
      }
      //Timer starten zum lesen der Daten
      akurventimer->start(100);
      idlezaehler = 0;
      //RS232 Empfangsbuffer leeren
      defaultlesentty();
      rxindex = 0;
      break;
    }
    break;
  case eeingang:
    switch(ksettings.htyp){
    case enpn:
    case epnp:
      //Basisstrom einstellen
      if(i10ua->isChecked())qs="@a0";
      if(i100ua->isChecked())qs="@a1";
      if(i1ma->isChecked())qs="@a2";
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      uc = getui1();
      uc = uc / uconstx;
      qs.sprintf("@c%04.0f",uc);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      icmax = (getui2() / iconsty);
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@e%04.0f",icmax);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      ekurventimer->start(100);
      idlezaehler = 0;
      defaultlesentty();
      rxindex = 0;
      break;
    case epmosfet:
    case enmosfet:
      if(rs232isopen){
        picmodem->writeLine("@a2");
      }else{
        warmstart();
      }
      uc = getui1() / uconstx;
      qs.sprintf("@c%04.0f",uc);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      icmax = getui2() / iconsty;
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@e%04.0f",icmax);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      ekurventimer->start(100);
      idlezaehler = 0;
      defaultlesentty();
      rxindex = 0;
      break;
    case enjfet:
    case epjfet:
      if(ksettings.htyp == enjfet){
        if(rs232isopen){
          picmodem->writeLine("@p2");
        }else{
          warmstart();
        }
      }
      if(ksettings.htyp == epjfet){
        if(rs232isopen){
          picmodem->writeLine("@p1");
        }else{
          warmstart();
        }
      }
      if(rs232isopen){
        picmodem->writeLine("@a2");//Ib = 1mA
      }else{
        warmstart();
      }
      uc = getui1() / uconstx;
      qs.sprintf("@c%04.0f",uc);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      icmax = getui2() / iconsty;
      icmax = icmax - double(kadcoffset);
      qs.sprintf("@f%04.0f",icmax);
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      ekurventimer->start(100);
      idlezaehler = 0;
      defaultlesentty();
      rxindex = 0;
      break;
    case ediode:
      if(i10ua->isChecked())qs="@a0";
      if(i100ua->isChecked())qs="@a1";
      if(i1ma->isChecked())qs="@a2";
      //Ib-Bereich umschalten
      if(rs232isopen){
        picmodem->writeLine(qs);
      }else{
        warmstart();
      }
      //qDebug(qPrintable(qs));
      if(rs232isopen){
        picmodem->writeLine("@g");
      }else{
        warmstart();
      }
      //Timer starten zum lesen der Daten
      akurventimer->start(100);
      idlezaehler = 0;
      //RS232 Empfangsbuffer leeren
      defaultlesentty();
      rxindex = 0;
      break;
    }
    break;
  }
}

void kschreiber::readakurventimer(){
  int gelesen = 0;
  unsigned char abuffer[2050];
  unsigned char c1, c2;
  int i, j, k, l=0;
//  bool kurvenende = false;
//  const double uconst = 10.0/kadcmax;
//  const double iconst = 500.0/kadcmax;
  double ub, xconst=0.0, yconst=0.0, messd1=0.0, messd2=0.0, ib;
  QString qs;
  QString strich = "-------------------------";

  kurvenende = false;
  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, 2050);
    if(gelesen > 0){
      for(i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
    }else{
      if(idlezaehler > 10){
        warmstart();
        return;
      }else{
        idlezaehler++;
      }
    }
    //Test ob das Ende der Datenuebertragung erreicht wurde
    for(i=0; i<2050; i+=2){
      c1 = rxbuffer[i];
      c2 = rxbuffer[i+1];
      if(int(uchar(c1) + uchar(c2) * 256) == 0x8000){
        kurvenende = true;
        //l = Anzahl der Messpunkte *2
        l = i - 2;
        //qDebug()<<"Kurvenende erkannt";
      }
      if(i == 2050){
        kurvenende = true;
        //l = Anzahl der Messpunkte
        l = i - 2;
      }
      //qDebug("i: %i, wert %i",i,int(c1 + c2 * 256));
    }
    //qDebug("l:%i",l);
    //Kurvenende erreicht
    if(kurvenende){
      //Ucollector Udrain auf 0 Volt eintstellen
      picmodem->writeLine("@c0000");
      //Timer RS232 auslesen stoppen
      akurventimer->stop();
      //Ubasis Ugate auf 0 Volt eintstellen
      picmodem->writeLine("@b0000");
      int messint[1024];
      //Array auf 0 setzen
      for(i=0;i<1024;i++)messint[i]=0;
      //Werte von uchar auf integer umspeichern
      for(j=0, i=0; j<l; j+=2, i++){
        c1 = rxbuffer[j];
        c2 = rxbuffer[j+1];
        messint[i] = adc2int(c1,c2);
        //messint[i] = int(uchar(c1) + uchar(c2) * 256 + kadcoffset);
        //if(i==1000)qDebug("messint[%i] %i",i,messint[i]);
        //qDebug()<<c1<<","<<c2;
      }
      j=l/2;
      k=0;//Messpunkt zaehler
      switch(ksettings.htyp){
        case enpn:
        case epnp:
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          xconst = 10.0 / 1023.0;
          yconst = 500.0 / kadcmax; //AD 10Bit oder ADC 12 Bit
        break;
        case ediode:
          xconst = getIb() / 1023.0;
          yconst = 10.0 / kadcmax; //AD 10Bit oder ADC 12 Bit
        break;
      }
      for(i=0; i<j; i++){
        messkurve.append(QPointF(double(k)*xconst,double(messint[i])*yconst));
        k++;
      }
      k--;
      messd1 = double(messint[k/2])*yconst;
      messd2 = double(messint[k])*yconst;
      //qDebug("messd1:%f",messd1);
      //qDebug("messd2:%f",messd2);
      textedit->clear();
      ib = 10.0;
      if(i100ua->isChecked())ib = 100.0;
      if(i1ma->isChecked())ib = 1000.0;
      if(ksettings.htyp == ediode){
        qs.sprintf("If1: %1.0f uA",ib/2.0);
        textedit->append(qs);
        qs.sprintf("Vf1: %1.3f V",messd1);
        textedit->append(qs);
        textedit->append(strich);
        qs.sprintf("If2: %1.0f uA",ib);
        textedit->append(qs);
        qs.sprintf("Vf2: %1.3f V",messd2);
        textedit->append(qs);
        textedit->append(strich);
      }
      ub = getui1();
      qs.sprintf("%1.2f",ub);
      if(ub >= 1.0)qs.sprintf("%1.2f",ub);
      if(ub >= 10.0)qs.sprintf("%1.1f",ub);
      if(ub >= 100.0)qs.sprintf("%1.0f",ub);
      if(ksettings.htyp == ediode)qs="";
      grafik->setCurveData(kurvennummer, messkurve, qs);
      kurvennummer++;
      if(checkaufzeichnung->isChecked()){
        qs.sprintf("%i;%f;%f",sphnr->value(),ub,double(messd2)*yconst);
        qs.replace('.',',');
        textedit->append(qs);
        messreihe << qs;
        int idx = sphnr->value();
        idx++;
        sphnr->setValue(idx);
      }
      text2info();
      grafik->setInfomessung(infomessung);
    }else{
      akurventimer->start(100);
    }
  }else{
    warmstart();
  }
}

void kschreiber::readekurventimer(){
  int gelesen = 0; //Anzahl der gelesenen Byte
  unsigned char abuffer[4098]; //temp lesebuffer RS232
  unsigned char c1, c2;
  int i, j, k, l=0;
  int messint;
//  bool kurvenende = false;
  double uconst = 10.0/1023.0;
  double uconstjfet = 10.0/1023.0;
  double iconst = 500.0/kadcmax;
  double uc;
  double icdelta, icdelta1, icdelta2, icdelta3;
  double ugdelta1=0.0, ugdelta2=0.0;
  double st;
  double uth = 0.0;
  QString qs;
  QString strich = "-------------------------";
  int readmax=0;
  
  //Festlegen der Datenmenge die maximal zurueck kommt
  kurvenende = false;
  switch(ksettings.htyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case ediode:
      readmax = 2050;
    break;
    case enjfet:
    case epjfet:
      readmax = 4098;
    break;
  }
  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, readmax);
    if(gelesen > 0){
      for(i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
    }else{
      if(idlezaehler > 10){
        warmstart();
        return;
      }else{
        idlezaehler++;
      }
    }
    //qDebug("ki %i",rxindex);
    //qDebug("gelesen %i",gelesen);
    for(i=0; i<readmax; i+=2){
      c1 = rxbuffer[i];
      c2 = rxbuffer[i+1];
      if(int(c1 + c2 * 256) == 0x8000){
        kurvenende = true;
        l = i - 2;
      }
      if(i == readmax){
        kurvenende = true;
        l = i - 2;
      }
      //qDebug("i: %i, wert %i",i,int(c1 + c2 * 256));
    }
    if(kurvenende){
      //Ucollector Udrain auf 0 Volt eintstellen
      if(rs232isopen)picmodem->writeLine("@c0000");
      //Basis/Gate einstellen
      switch(ksettings.htyp){
      case enpn:
      case enmosfet:
      case ediode:
        if(rs232isopen)picmodem->writeLine("@p0");
        if(rs232isopen)picmodem->writeLine("@b0000");
        break;
      case epnp:
      case epmosfet:
        if(rs232isopen)picmodem->writeLine("@p3");
        if(rs232isopen)picmodem->writeLine("@b0000");
        break;
      case enjfet:
        if(rs232isopen)picmodem->writeLine("@p2");
        if(rs232isopen)picmodem->writeLine("@b1023");
       break;
      case epjfet:
        if(rs232isopen)picmodem->writeLine("@p1");
        if(rs232isopen)picmodem->writeLine("@b1023");
       break;
      }
      c1 = rxbuffer[l-2];
      c2 = rxbuffer[l-1];
      messint = adc2int(c1,c2);
      //messint = int(c1 + c2 * 256 + kadcoffset);
      //icdelta2 ist letzter Ic Wert 
      icdelta2 = messint * iconst;
      //icdelta3 ist Ic halbe
      icdelta3 = icdelta2 / 2.0;
      switch(ksettings.htyp){
        case epnp:
        case enpn:
        case ediode:
          ugdelta2 = double(l/2) * getIbconst();
        break;
        case enmosfet:
        case epmosfet:
          ugdelta2 = double(l / 2) * uconst;
        break;
        case enjfet:
        case epjfet:
          ugdelta2 = (double(l / 2) * uconstjfet) - 10.0;
        break;
      }
      icdelta1 = 0.0;
      //qDebug("l %i",l);
      int messintf[2048];
      for(i=0;i<2048;i++)messintf[i]=0;
      for(j=0, i=0; j<l; j+=2, i++){
          c1 = rxbuffer[j];
          c2 = rxbuffer[j+1];
          messintf[i] = adc2int(c1,c2);
          //messintf[i] = int(c1 + c2 * 256 + kadcoffset);
          //qDebug("messintf %i",messintf[i]);
      }
      j=l/2;
      k=0;
      //Uth ermitteln
      for(i=0; i<j; i++){
          if((uth == 0.0) and (messintf[i] > 0.0)){
          switch(ksettings.htyp){
            case epnp:
            case enpn:
            case ediode:
            break;
            case enmosfet:
            case epmosfet:
              uth = double(k) * uconst;
            break;
            case enjfet:
            case epjfet:
              uth = (double(k) * uconstjfet) - 10.0;
            break;
          }
        }
        icdelta = double(messintf[i]) * iconst;
        if(icdelta3 < icdelta){
          if(icdelta1 == 0.0){
            icdelta1 = icdelta;
            switch(ksettings.htyp){
              case epnp:
              case enpn:
              case ediode:
                ugdelta1 = double(k) * getIbconst();
              break;
              case enmosfet:
              case epmosfet:
                ugdelta1 = double(k) * uconst;
              break;
              case enjfet:
              case epjfet:
                ugdelta1 = (double(k) * uconstjfet) - 10.0;
              break;
            }
          }
        }
        //qDebug("messd %i",messd);
        double ibgconst = getIbconst();
        switch(ksettings.htyp){
          case enpn:
          case epnp:
            messkurve.append(QPointF(double(k)*ibgconst,double(messintf[i])*iconst));
          break;
          case ediode:
            messkurve.append(QPointF(double(k)*ibgconst,double(messintf[i])*iconst));
          break;
          case enmosfet:
          case epmosfet:
            messkurve.append(QPointF(double(k)*uconst,double(messintf[i])*iconst));
          break;
          case enjfet:
          case epjfet:
            messkurve.append(QPointF((double(k)*uconstjfet)-10.0,double(messintf[i])*iconst));
          break;
        }
        k++;
      }
      textedit->clear();
      switch(ksettings.htyp){
        case enpn:
        case epnp:
          qs.sprintf("Ib1: %2.3f uA",ugdelta1);
          textedit->append(qs);
          qs.sprintf("Ic1: %2.3f mA",icdelta1);
          textedit->append(qs);
          st = icdelta1 / (ugdelta1 / 1000.0);
          qs.sprintf("hfe1: %3.0f", st);
          textedit->append(qs);
          textedit->append(strich);
          qs.sprintf("Ib2: %2.3f uA",ugdelta2);
          textedit->append(qs);
          qs.sprintf("Ic2: %2.3f mA",icdelta2);
          textedit->append(qs);
          st = icdelta2 / (ugdelta2 / 1000.0);
          qs.sprintf("hfe2: %3.0f", st);
          textedit->append(qs);
          textedit->append(strich);
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          qs.sprintf("Uth: %2.3f V",uth);
          textedit->append(qs);
          textedit->append(strich);
          qs.sprintf("Ic1: %2.3f mA",icdelta1);
          textedit->append(qs);
          qs.sprintf("Ic2: %2.3f mA",icdelta2);
          textedit->append(qs);
          qs.sprintf("Ug1: %2.3f V",ugdelta1);
          textedit->append(qs);
          qs.sprintf("Ug2: %2.3f V",ugdelta2);
          textedit->append(qs);
          st = ((icdelta2 - icdelta1)/1000.0) / (ugdelta2 - ugdelta1);
          if(st < 1.0){
            qs.sprintf(" %3.0f mS",st*1000.0);
            qs = tr("Steilheit:") + qs;
          }else{
            qs.sprintf(" %2.3f S",st);
            qs = tr("Steilheit:") + qs;
          }
          textedit->append(qs);
          textedit->append(strich);
        break;
        case ediode:
        break;
      }
      if(checkaufzeichnung->isChecked()){
        switch(ksettings.htyp){
          case enpn:
          case epnp:
            qs.sprintf("%i;%f;%f;%f;%f;%f;%f",sphnr->value()
                                             ,ugdelta1,icdelta1,icdelta1 / (ugdelta1 / 1000.0)
                                             ,ugdelta2,icdelta2,icdelta2 / (ugdelta2 / 1000.0));
          break;
          case enmosfet:
          case epmosfet:
          case enjfet:
          case epjfet:
            qs.sprintf("%i;%f;%f;%f;%f;%f;%f",sphnr->value()
                                          ,uth,ugdelta1,icdelta1,ugdelta2,icdelta2
                                          ,((icdelta2 - icdelta1)/1000.0) / (ugdelta2 - ugdelta1));
          break;
          case ediode:
          break;
        }
        qs.replace('.',',');
        textedit->append(qs);
        messreihe << qs;
        int idx = sphnr->value();
        idx++;
        sphnr->setValue(idx);
      }
      uc = getui1();
      QString qs;
      if(uc >= 1.0)qs.sprintf("%1.2f",uc);
      if(uc >= 10.0)qs.sprintf("%1.1f",uc);
      if(uc >= 100.0)qs.sprintf("%1.0f",uc);
      grafik->setCurveData(kurvennummer, messkurve, qs);
      ekurventimer->stop();
      kurvennummer++;
      //qDebug("kadcmax: %f",kadcmax);
      text2info();
      grafik->setInfomessung(infomessung);
    }else{
      ekurventimer->start(100);
    }
  }else{
    warmstart();
  }
}

void kschreiber::actrepainttimer(){
  grafiksetzen();
  resize(this->rect().size());
  repainttimer->stop();
  //qDebug("kschreiber::actrepainttimer()");
}

void kschreiber::clickbclearcurven(){
  textedit->clear();
  grafik->loeschenAlleKurven();
  kurvennummer = 0;
}

void kschreiber::clickbsethintergrund(){
  grafik->Kurven2Hintergrund();
  grafik->loeschenAlleKurven();
  kurvennummer = 0;
}

void kschreiber::clickbdastart(){
  if(rs232isopen){
    picmodem->writeLine("@d");
  }else{
    warmstart();
  }
}

void kschreiber::setPolaritaet(){
  int p;
  QString qs;
  
  if((ubpol == 1.0) and (ucpol == 1.0))  p = 0;
  if((ubpol == 1.0) and (ucpol == -1.0)) p = 1;
  if((ubpol == -1.0) and (ucpol == 1.0)) p = 2;
  if((ubpol == -1.0) and (ucpol == -1.0))p = 3;
  if(rs232isopen){
    qs.sprintf("@p%1i",p);
    //qDebug()<<"setPolarität:"<<qs;
    picmodem->writeLine(qs); //Polaritaeten Ub Uc einstellen
  }else{
    warmstart();
  }
}

void kschreiber::clickaufzeichnung(int status){
  QFile fmess;
  QString qs;
  QString s,s1;
  

  if(status == Qt::Checked){
    messreihe.clear();
    if(kurventyp == eausgang){
      switch(ksettings.htyp){
        case enpn:
        case epnp:
          messreihe << "ObjNr;Ib;Ic";
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          messreihe << "ObjNr;Ug;Id";
        break;
        case ediode:
        break;
      }
    }else{
      switch(ksettings.htyp){
        case enpn:
        case epnp:
          messreihe << "ObjNr;Ib1;Ic1;hfe1;Ib2;Ic2;hfe2";
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          messreihe << "ObjNr;Uth;Ug1;Ic1;Ug2;Ic2;S";
        break;
        case ediode:
        break;
      }
    }
    //qDebug("Start Aufzeichnung");
  }
  if(status == Qt::Unchecked){
    s = edateiname->text();
    if(!s.contains(".")) s += ".csv";
    fmess.setFileName(s);
    while(fmess.exists()){
      s1 += "$";
      fmess.setFileName(s+s1);
    }
    if(fmess.open(QIODevice::WriteOnly)){
      QTextStream ts(&fmess);
      for(int i=0; i<messreihe.size();i++){
        ts << messreihe.at(i) << endl;
      }
    }
    fmess.close();
    //qDebug("Stop Aufzeichnung");
  }
}

void kschreiber::aendernboxui1(){
  bool ok;
  
  switch(ksettings.htyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case enjfet:
    case epjfet:
    break;
    case ediode:
      QString qs = boxui1->currentText();
      double wert = qs.toDouble(&ok);
      ksettings.setminY(0.0);
      ksettings.setmaxY(wert);
      ksettings.setnumYTicks(20);
      grafik->setPlotSettings(ksettings);
    break;
  }
}

double kschreiber::getui1const(){
  double wert = getui1()/kadcmax;
  return wert;
}

double kschreiber::getui1(){
  bool ok;
  
  int index = boxui1->currentIndex();
  QString qs = boxui1->itemText(index);
  //qDebug()<<qs;
  double wert = qs.toDouble(&ok);
  if(ok){
    return wert;
  }else{
    return 0.0;
  }
}

void kschreiber::aendernboxui2(int index){
  bool ok;
  QString qs;
  double wert;
  
  switch(ksettings.htyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case enjfet:
    case epjfet:
      qs = boxui2->itemText(index);
      wert = qs.toDouble(&ok);
      ksettings.setminY(0.0);
      ksettings.setmaxY(wert);
      ksettings.setnumYTicks(20);
      grafik->setPlotSettings(ksettings);
    break;
    case ediode:
    break;
  }
}

double kschreiber::getui2const(){
  double wert = getui2()/kadcmax;
  return wert;
}

double kschreiber::getui2(){
  bool ok;
  
  int index = boxui2->currentIndex();
  QString qs = boxui2->itemText(index);
  double wert = qs.toDouble(&ok);
  if(ok){
    return wert;
  }else{
    return 0.0;
  }
}

void kschreiber::aendernIb(){
  double wert;
  
  switch(kurventyp){
    case eeingang:
      switch(ksettings.htyp){
        case enpn:
        case epnp:
        case ediode:
          if(i10ua->isChecked()){
            wert=10.0;
          }
          if(i100ua->isChecked()){
            wert=100.0;
          }
          if(i1ma->isChecked()){
            wert=1000.0;
          }
          ksettings.setminX(0.0);
          ksettings.setmaxX(wert);
          ksettings.setnumXTicks(20);
          grafik->setPlotSettings(ksettings);
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
        break;
      }
    break;
    case eausgang:
      switch(ksettings.htyp){
        case ediode:
          if(i10ua->isChecked()){
            wert=10.0;
          }
          if(i100ua->isChecked()){
            wert=100.0;
          }
          if(i1ma->isChecked()){
            wert=1000.0;
          }
          ksettings.setminX(0.0);
          ksettings.setmaxX(wert);
          ksettings.setnumXTicks(20);
          grafik->setPlotSettings(ksettings);
        break;
        case enpn:
        case epnp:
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          boxui1fuellen();
        break;
      }
    break;
  }
}

double kschreiber::getIbconst(){
  double wert = getIb()/1023.0;
  return wert;
}

double kschreiber::getIb(){
  double wert=0.0;
  
  if(i10ua->isChecked()){
    wert=10.0;
  }
  if(i100ua->isChecked()){
    wert=100.0;
  }
  if(i1ma->isChecked()){
    wert=1000.0;
  }
  return wert;
}

void kschreiber::boxui1fuellen(){
  QString qs;
  double w;
  int i;
  
  boxui1->clear();
  switch(kurventyp){
  case eausgang:
    switch(ksettings.htyp){
      case enpn:
      case epnp:
        if(i10ua->isChecked()){
          w = 10.0;
          for(i=1;i<20;i++){
            qs.sprintf("%2.1f",w);
            boxui1->addItem(qs);
            w = w - 0.5;
          }
        }
        if(i100ua->isChecked()){
          w = 100.0;
          for(i=1;i<20;i++){
            qs.sprintf("%2.0f",w);
            boxui1->addItem(qs);
            w = w - 5.0;
          }
        }
        if(i1ma->isChecked()){
          w = 1000.0;
          for(i=1;i<20;i++){
            qs.sprintf("%2.0f",w);
            boxui1->addItem(qs);
            w = w - 50.0;
          }
        }
      break;
      case enmosfet:
      case epmosfet:
        w = 10.0;
        for(i=1;i<=100;i++){
          qs.sprintf("%2.1f",w);
          boxui1->addItem(qs);
          w = w - 0.1;
        }
      break;
      case enjfet:
      case epjfet:
        w = 10.0;
        for(i=1;i<=101;i++){
          qs.sprintf("%2.1f",w);
          boxui1->addItem(qs);
          w = w - 0.2;
        }
      break;
      case ediode:
        w = 10.0;
        for(i=1;i<=20;i++){
          qs.sprintf("%2.1f",w);
          boxui1->addItem(qs);
          w = w - 0.5;
        }
      break;
    }
    break;
    case eeingang:
      switch(ksettings.htyp){
        case enpn:
        case epnp:
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          w = 10.0;
          for(i=1;i<=100;i++){
            qs.sprintf("%2.1f",w);
            boxui1->addItem(qs);
            w = w - 0.1;
          }
        break;
        case ediode:
          w = 10.0;
          for(i=1;i<=20;i++){
            qs.sprintf("%2.1f",w);
            boxui1->addItem(qs);
            w = w - 0.5;
          }
        break;
      }
    break;
  }
}

void kschreiber::boxui2fuellen(){
  
  boxui2->clear();
  switch(ksettings.htyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case enjfet:
    case epjfet:
      boxui2->addItem("500");
      boxui2->addItem("400");
      boxui2->addItem("300");
      boxui2->addItem("200");
      boxui2->addItem("100");
      boxui2->addItem("90");
      boxui2->addItem("80");
      boxui2->addItem("70");
      boxui2->addItem("60");
      boxui2->addItem("50");
      boxui2->addItem("40");
      boxui2->addItem("30");
      boxui2->addItem("20");
      boxui2->addItem("10");
    break;
    case ediode:
    break;
  }
}

void kschreiber::cfgladen(){
  configfile cfgk;
  int r,g,b,a;
  QColor color;
  QDir homepath=QDir::homePath();

  cfgk.open(homepath.filePath(".ksch.cfg"));
  ucdeinnpn = cfgk.readDouble("ucd_ein_npn", 7.0);
  ucdeinpnp = cfgk.readDouble("ucd_ein_pnp", 7.0);
  ucdeinnmos = cfgk.readDouble("ucd_ein_nmos", 7.0);
  ucdeinpmos = cfgk.readDouble("ucd_ein_pmos", 7.0);
  ucdeinnjfet = cfgk.readDouble("ucd_ein_njfet", 7.0);
  ucdeinpjfet = cfgk.readDouble("ucd_ein_pjfet", 7.0);
  udiode = cfgk.readDouble("u_diode", 1.0);
  ibausnpn = cfgk.readDouble("ib_aus_npn", 200.0);
  ibauspnp = cfgk.readDouble("ib_aus_pnp", 200.0);
  ibausdiode = cfgk.readDouble("ib_aus_diode", 1000.0);
  ibeinnpn = cfgk.readDouble("ib_ein_npn", 10.0);
  ibeinpnp = cfgk.readDouble("ib_ein_pnp", 10.0);
  ibeindiode = cfgk.readDouble("ib_ein_diode", 1000.0);
  ugausnmos = cfgk.readDouble("ug_aus_nmos", 4.0);
  ugauspmos = cfgk.readDouble("ug_aus_pmos", 4.0);
  ugausnjfet = cfgk.readDouble("ug_aus_njfet", -1.0);
  ugauspjfet = cfgk.readDouble("ug_aus_pjfet", -1.0);
  imaxnpn = cfgk.readDouble("imax_npn", 100.0);
  imaxpnp = cfgk.readDouble("imax_pnp", 100.0);
  imaxnmos = cfgk.readDouble("imax_nmos", 200.0);
  imaxpmos = cfgk.readDouble("imax_pmos", 200.0);
  imaxnjfet = cfgk.readDouble("imax_njfet", 20.0);
  imaxpjfet = cfgk.readDouble("imax_pjfet", 20.0);
  r = cfgk.readInteger("hintergrundr", 255);
  g = cfgk.readInteger("hintergrundg", 255);
  b = cfgk.readInteger("hintergrundb", 152);
  a = cfgk.readInteger("hintergrunda", 255);
  colorhintergrund = QColor(r,g,b,a);
  ksettings.setcolorhintergrund(colorhintergrund);

  //Button einfaerben
  QPalette palette = bcolorhintergrund->palette();
  palette.setColor(QPalette::Active, QPalette::Button, colorhintergrund);
  bcolorhintergrund->setPalette(palette);
  r = cfgk.readInteger("schriftr", 118);
  g = cfgk.readInteger("schriftg", 117);
  b = cfgk.readInteger("schriftb", 136);
  a = cfgk.readInteger("schrifta", 255);
  colorschrift = QColor(r,g,b,a);
  ksettings.setcolorschrift(colorschrift);
  r = cfgk.readInteger("ueberschriftr", 0);
  g = cfgk.readInteger("ueberschriftg", 0);
  b = cfgk.readInteger("ueberschriftb", 0);
  a = cfgk.readInteger("ueberschrifta", 255);
  ueberschriftsize = cfgk.readInteger("ueberschriftsize", 16);
  colorueberschrift = QColor(r,g,b,a);
  ksettings.setcolorueberschrift(colorueberschrift, ueberschriftsize);
  r = cfgk.readInteger("clinier", 255);
  g = cfgk.readInteger("clinieg", 0);
  b = cfgk.readInteger("clinieb", 0);
  a = cfgk.readInteger("cliniea", 255);
  colorlinie = QColor(r,g,b,a);
  ksettings.setcolorlinie(colorlinie);
  r = cfgk.readInteger("cgitterr", 186);
  g = cfgk.readInteger("cgitterg", 186);
  b = cfgk.readInteger("cgitterb", 186);
  a = cfgk.readInteger("cgittera", 255);
  colorgitter = QColor(r,g,b,a);
  ksettings.setcolorgitter(colorgitter);
  int fsize = cfgk.readInteger("liniefontsize", 9);
  ksettings.setliniefontsize(fsize);
  sboxliniefsize->setValue(fsize);
  fsize = cfgk.readInteger("messinfofontsize", 12);
  ksettings.setlfontsize(fsize);
  sboxliniesize->setValue(fsize);
  r = cfgk.readInteger("chlinier", 0);
  g = cfgk.readInteger("chlinieg", 0);
  b = cfgk.readInteger("chlinieb", 255);
  a = cfgk.readInteger("chliniea", 255);
  colorhlinie = QColor(r,g,b,a);
  ksettings.setcolorhlinie(colorhlinie);
  kadcoffset = cfgk.readInteger("adcoffset",0);
  spadcoffset->setValue(kadcoffset);
  cfgk.close();
}

void kschreiber::cfgspeichern(){
  configfile cfgk;
  int r,g,b,a;
  QColor color;
  QDir homepath=QDir::homePath();

  cfgk.open(homepath.filePath(".ksch.cfg"));
  cfgk.writeDouble("ucd_ein_npn", ucdeinnpn);
  cfgk.writeDouble("ucd_ein_pnp", ucdeinpnp);
  cfgk.writeDouble("ucd_ein_nmos", ucdeinnmos);
  cfgk.writeDouble("ucd_ein_pmos", ucdeinpmos);
  cfgk.writeDouble("ucd_ein_njfet", ucdeinnjfet);
  cfgk.writeDouble("ucd_ein_pjfet", ucdeinpjfet);
  cfgk.writeDouble("u_diode", udiode);
  cfgk.writeDouble("ib_aus_npn", ibausnpn);
  cfgk.writeDouble("ib_aus_pnp", ibauspnp);
  cfgk.writeDouble("ib_aus_diode", ibausdiode);
  cfgk.writeDouble("ib_ein_npn", ibeinnpn);
  cfgk.writeDouble("ib_ein_pnp", ibeinpnp);
  cfgk.writeDouble("ib_ein_diode", ibeindiode);
  cfgk.writeDouble("ug_aus_nmos", ugausnmos);
  cfgk.writeDouble("ug_aus_pmos", ugauspmos);
  cfgk.writeDouble("ug_aus_njfet", ugausnjfet);
  cfgk.writeDouble("ug_aus_pjfet", ugauspjfet);
  cfgk.writeDouble("imax_npn", imaxnpn);
  cfgk.writeDouble("imax_pnp", imaxpnp);
  cfgk.writeDouble("imax_nmos", imaxnmos);
  cfgk.writeDouble("imax_pmos", imaxpmos);
  cfgk.writeDouble("imax_njfet", imaxnjfet);
  cfgk.writeDouble("imax_pjfet", imaxpjfet);
  colorhintergrund.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("hintergrundr", r);
  cfgk.writeInteger("hintergrundg", g);
  cfgk.writeInteger("hintergrundb", b);
  cfgk.writeInteger("hintergrunda", a);
  colorschrift.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("schriftr", r);
  cfgk.writeInteger("schriftg", g);
  cfgk.writeInteger("schriftb", b);
  cfgk.writeInteger("schrifta", a);
  colorueberschrift.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("ueberschriftr", r);
  cfgk.writeInteger("ueberschriftg", g);
  cfgk.writeInteger("ueberschriftb", b);
  cfgk.writeInteger("ueberschrifta", a);
  colorlinie.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("clinier", r);
  cfgk.writeInteger("clinieg", g);
  cfgk.writeInteger("clinieb", b);
  cfgk.writeInteger("cliniea", a);
  colorgitter.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("cgitterr", r);
  cfgk.writeInteger("cgitterg", g);
  cfgk.writeInteger("cgitterb", b);
  cfgk.writeInteger("cgittera", a);
  int fsize = ksettings.liniefontsize();
  cfgk.writeInteger("liniefontsize", fsize);
  fsize = ksettings.getlfontsize();
  cfgk.writeInteger("messinfofontsize", fsize);
  colorhlinie.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("chlinier", r);
  cfgk.writeInteger("chlinieg", g);
  cfgk.writeInteger("chlinieb", b);
  cfgk.writeInteger("chliniea", a);
  cfgk.writeInteger("adcoffset",kadcoffset);
  cfgk.close();
}

void kschreiber::getUI(){
  double ui1;
  double ui2;
  double ib;
  bool ok;
  QString qs;
  
  qs = boxui1->currentText();
  ui1 = qs.toDouble(&ok);
  qs = boxui2->currentText();
  ui2 = qs.toDouble(&ok);
  ib = 10.0;
  if(i100ua->isChecked())ib = 100.0;
  if(i1ma->isChecked())ib = 1000.0;
  switch(kurventyp){
    case eeingang:
      switch(ksettings.htyp){
        case enpn:
          ucdeinnpn = ui1;
          imaxnpn = ui2;
          ibeinnpn = ib;
        break;
        case epnp:
          ucdeinpnp = ui1;
          imaxpnp = ui2;
          ibeinpnp = ib;
        break;
        case enmosfet:
          ucdeinnmos = ui1;
          imaxnmos = ui2;
        break;
        case epmosfet:
          ucdeinpmos = ui1;
          imaxpmos = ui2;
        break;
        case enjfet:
          ucdeinnjfet = ui1;
          imaxnjfet = ui2;
        break;
        case epjfet:
          ucdeinpjfet = ui1;
          imaxpjfet = ui2;
        break;
        case ediode:
          udiode = ui1;
          ibeindiode = ib;
        break;
      }
    break;
    case eausgang:
      switch(ksettings.htyp){
        case enpn:
          ibausnpn = ui1;
          imaxnpn = ui2;
        break;
        case epnp:
          ibauspnp = ui1;
          imaxpnp = ui2;
        break;
        case enmosfet:
          ugausnmos = ui1;
          imaxnmos = ui2;
        break;
        case epmosfet:
          ugauspmos = ui1;
          imaxpmos = ui2;
        break;
        case enjfet:
          ugausnjfet = ui1;
          imaxnjfet = ui2;
        break;
        case epjfet:
          ugauspjfet = ui1;
          imaxpjfet = ui2;
        break;
        case ediode:
          udiode = ui1;
          ibausdiode = ib;
        break;
      }
    break;
  }
}

void kschreiber::setUI(){
  double ui1=0.0;
  double ui2=0.0;
  double ib=0.0;
  bool ok;
  QString qs;
  int auswahl;
  
  switch(kurventyp){
    case eeingang:
      switch(ksettings.htyp){
        case enpn:
          ui1 = ucdeinnpn;
          ui2 = imaxnpn;
          ib = ibeinnpn;
        break;
        case epnp:
          ui1 = ucdeinpnp;
          ui2 = imaxpnp;
          ib = ibeinpnp;
        break;
        case enmosfet:
          ui1 = ucdeinnmos;
          ui2 = imaxnmos;
        break;
        case epmosfet:
          ui1 = ucdeinpmos;
          ui2 = imaxpmos;
        break;
        case enjfet:
          ui1 = ucdeinnjfet;
          ui2 = imaxnjfet;
        break;
        case epjfet:
          ui1 = ucdeinpjfet;
          ui2 = imaxpjfet;
        break;
        case ediode:
          ui1 = udiode;
          ib = ibeindiode;
        break;
      }
      auswahl = 0;
      if(ib > 10.0)auswahl = 1;
      if(ib > 100.0)auswahl = 2;
      switch(auswahl){
        case 0:
          i10ua->setChecked(true);
          clickb10uA();
        break;
        case 1:
          i100ua->setChecked(true);
          clickb100uA();
        break;
        case 2:
          i1ma->setChecked(true);
          clickb1mA();
        break;
      }
    break;
    case eausgang:
      switch(ksettings.htyp){
        case enpn:
          ui1 = ibausnpn;
          auswahl = 0;
          if(ui1 > 10.0)auswahl = 1;
          if(ui1 > 100.0)auswahl = 2;
          switch(auswahl){
            case 0:
              i10ua->setChecked(true);
              clickb10uA();
            break;
            case 1:
              i100ua->setChecked(true);
              clickb100uA();
            break;
            case 2:
              i1ma->setChecked(true);
              clickb1mA();
            break;
          }
          ui2 = imaxnpn;
        break;
        case epnp:
          ui1 = ibauspnp;
          i10ua->setChecked(true);
          clickb10uA();
          if(ui1 > 10.0){
            i100ua->setChecked(true);
            clickb100uA();
          }
          if(ui1 > 100.0){
            i1ma->setChecked(true);
            clickb1mA();
          }
          ui2 = imaxpnp;
        break;
        case enmosfet:
          ui1 = ugausnmos;
          ui2 = imaxnmos;
        break;
        case epmosfet:
          ui1 = ugauspmos;
          ui2 = imaxpmos;
        break;
        case enjfet:
          ui1 = ugausnjfet;
          ui2 = imaxnjfet;
        break;
        case epjfet:
          ui1 = ugauspjfet;
          ui2 = imaxpjfet;
        break;
        case ediode:
          ui1 = udiode;
          ib = ibausdiode;
          i10ua->setChecked(true);
          clickb10uA();
          if(ib > 10.0){
            i100ua->setChecked(true);
            clickb100uA();
          }
          if(ib > 100.0){
            i1ma->setChecked(true);
            clickb1mA();
          }
        break;
      }
    break;
  }
  for(int i=0; i < boxui1->count();i++){
    boxui1->setCurrentIndex(i);
    qs = boxui1->itemText(i);
    if(ui1 == qs.toDouble(&ok))break;
  }
  for(int i=0; i < boxui2->count();i++){
    boxui2->setCurrentIndex(i);
    qs = boxui2->itemText(i);
    if(ui2 == qs.toDouble(&ok))break;
  }
}

void kschreiber::clickcolorhintergrund(){
  bool ok;
  QColor color = colorhintergrund;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorhintergrund = color;
    ksettings.setcolorhintergrund(colorhintergrund);
    setKurvenSchreiber();
    QPalette palette = bcolorhintergrund->palette();
    palette.setColor(QPalette::Active, QPalette::Button, color);
    bcolorhintergrund->setPalette(palette);
  }
}

void kschreiber::clickcolorschrift(){
  bool ok;
  QColor color = colorschrift;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorschrift = color;
    ksettings.setcolorschrift(colorschrift);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorueberschrift(){
  bool ok;
  QColor color = colorueberschrift;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorueberschrift = color;
    ksettings.setcolorueberschrift(colorueberschrift,ueberschriftsize);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorlinie(){
  bool ok;
  QColor color = colorlinie;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorlinie = color;
    ksettings.setcolorlinie(colorlinie);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorgitter(){
  bool ok;
  QColor color = colorgitter;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorgitter = color;
    ksettings.setcolorgitter(colorgitter);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorhlinie(){
  bool ok;
  QColor color = colorhlinie;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorhlinie = color;
    ksettings.setcolorhlinie(colorhlinie);
    setKurvenSchreiber();
  }
}

//QString kschreiber::punkt2komma(QString &qs){
//}

void kschreiber::fopentty(){
  int gelesen;
  
  if(bvantwort){
    geraet = ' ';
    ksettings.klsfwversion = 0;
    bvantwort = false;
    gelesen = picmodem->readttybuffer(rxbuffer, 2);
    if(gelesen == 2){
      geraet = rxbuffer[0];
      ksettings.klsfwversion = uchar(rxbuffer[1]);
      //qDebug("ksettings.klsfwversion: %i",ksettings.klsfwversion);
      bvantwort = false;
      //qDebug("gelesen: %i",gelesen);
      if((geraet == 'K')and(ksettings.klsfwversion > 100)){
        if(ksettings.klsfwversion >= 200){
          kadcmax=4092.0 + double(kadcoffset);
          spadcoffset->setEnabled(true);
          ladcoffset->setEnabled(true);
        }else{
          kadcoffset=0;
          spadcoffset->setEnabled(false);
          ladcoffset->setEnabled(false);
        }

        grafik->setPlotSettings(ksettings);
        bksgefunden = true;
        openttytimer->stop(); //Suchen der Schnittstelle bendet
        statusBar()->showMessage(tr("HW gefunden an: ") + srs232);
        setAText(srs232);
        awidget->setEnabled(true);
        bwstart = true;
      }else{
        idtty++;
        if(idtty > (listtty.count()-1))idtty = 0;
        openttytimer->start(200); //Suchen der Schnittstelle
        picmodem->closetty();
        rs232isopen = false;
      }
    }else{
      idtty++;
      if(idtty > (listtty.count()-1))idtty = 0;
      openttytimer->start(200); //Suchen der Schnittstelle
      picmodem->closetty();
      rs232isopen = false;
    }
  }else{
    if(!rs232isopen){
      srs232 = listtty.at(idtty);
      statusBar()->showMessage("Suche HW :" + srs232);
      //qDebug(qPrintable(srs232));
      //qDebug("%i",idtty);
      rs232isopen = picmodem->opentty(srs232);
      if(rs232isopen){
        defaultlesentty();
        picmodem->writeLine("@v");
        bvantwort = true;
        openttytimer->start(200); //Suchen der Schnittstelle
      }else{
        idtty++;
        if(idtty > (listtty.count()-1))idtty = 0;
        openttytimer->start(100); //Suchen der Schnittstelle
        bvantwort = false;
        picmodem->closetty();
      }
    }
  }
}

void kschreiber::warmstart(){
  if(bwstart){
    QMessageBox::critical(this, tr("Hardware nicht gefunden","WarnungsBox"), 
                                tr("<B>Die USB-Schnittstelle ist nicht zu finden!</B><BR>"
                                   "Die automatische Suche wird aktiviert!","WarnungsBox"));
    idtty=0;
    testtimer->stop();
    imaxtimer->stop();
    akurventimer->stop();
    ekurventimer->stop();
    repainttimer->stop();
    openttytimer->stop();
    picmodem->closetty();
    rs232isopen = false;
    awidget->setEnabled(false);
    ksettings.klsfwversion = 0;
    setAText("");
    openttytimer->start(200); //Suchen der Schnittstelle
  }
}

void kschreiber::setlinieFontSize(int i){
  QFont font=labelliniefsize->font();
  font.setBold(false);
  font.setPixelSize(i);
  labelliniefsize->setFont(font);
  ksettings.setliniefontsize(i);
  grafik->setPlotSettings(ksettings);
}

void kschreiber::setlFontSize(int i){
  QFont font=labelliniesize->font();
  font.setBold(false);
  font.setPixelSize(i);
  labelliniesize->setFont(font);
  ksettings.setlfontsize(i);
  grafik->setPlotSettings(ksettings);
}
void kschreiber::bildspeichern(){
  QString qs;
  qs=datetime2filename();
  qs=fnamenormal(qs);
  emit dbildspeichern(qs);
}

QString kschreiber::datetime2filename(){
  QString sdt;

  QDateTime dtime=QDateTime::currentDateTime();
  sdt=dtime.toString("yyMMdd_hhmmss");
  return sdt;
}

QString kschreiber::fnamenormal(QString &qs){
  QString s=qs;
  s.replace('/','_');
  s.replace(',','_');
  s.replace(':','_');
  s.replace('\\','_');
  s.replace(';','_');
  return s;
}

void kschreiber::labelspeichern(){
  bool ok;
  QString qs;

  //als Default das Datum eintragen
  if(ksettings.caption.isEmpty()){
    qs=datetime2filename();
  }else{
    qs=ksettings.caption;
  }

  QString text = QInputDialog::getText(this, tr("Plot Beschreibung"),
                                       tr("Beschreibung:"), QLineEdit::Normal,
                                       qs, &ok);
  if (ok && !text.isEmpty()){
    text.replace(' ', '_');
    ksettings.caption=text;//Caption in Messkurve eintragen
    grafik->setPlotSettings(ksettings);
  }
}

int kschreiber::adc2int(uchar c1, uchar c2){
  int i = c1 + c2*256 + kadcoffset;
  if(i<0)i=0;
  return i;
}

void kschreiber::setadcoffset(int off){
  kadcoffset=off;
  if(ksettings.klsfwversion >= 200){
    kadcmax = 4092.0 + double(kadcoffset);
  }else{
    kadcmax = 1023.0 + double(kadcoffset);
  }
}

void kschreiber::text2info(){
  //StringList loeschen
  infomessung.clear();
  //Alles als String auslesen
  QString data = textedit->toPlainText();
  //String in StringList wandeln
  infomessung = data.split(QRegExp("\n"),QString::SkipEmptyParts);
}

void kschreiber::setTextEdit(){
  text2info();
  grafik->setInfomessung(infomessung);
}

void kschreiber::empfmwert(QPointF data){
  QString qs, qs1, qs2;
  QString strich = "-------------------------";

  qs.sprintf("X: %1.2f ",data.x());
  if(data.x() >= 10.0)qs.sprintf("X: %1.1f ",data.x());
  if(data.x() >= 100.0)qs.sprintf("X: %1.0f ",data.x());
  qs1 = qs + ksettings.xmasseinheit;
  //qDebug()<<data.x();
  //qDebug()<<qs1;
  qs.sprintf("Y: %1.2f ",data.y());
  if(data.y() >= 10.0)qs.sprintf("Y: %1.1f ",data.y());
  if(data.y() >= 100.0)qs.sprintf("Y: %1.0f ",data.y());
  qs2 = qs + ksettings.ymasseinheit;
  textedit->append(qs1);
  textedit->append(qs2);
  textedit->append(strich);
  //qDebug()<<qs2;
}
