/***************************************************************************
                          wdisplay.cpp  -  description
                             -------------------
    begin                : 14 11 2009
    copyright            : (C) 2009 by Andreas Lindenau
    email                : DL4JAL@darc.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <cmath>
//#include <QtGui>
#include "wdisplay.h"

WDisplay::WDisplay(QWidget *parent) : QWidget(parent)
{
  popupmenu = new QMenu();
  ainfoset = new QAction(tr("Infotext einfuegen","PopupMenue"), popupmenu);
  connect(ainfoset, SIGNAL(triggered()), this, SLOT(infotextset()));
  popupmenu->addAction(ainfoset);
  ainfoclr = new QAction(tr("Infotext loeschen","PopupMenue"), popupmenu);
  connect(ainfoclr, SIGNAL(triggered()), this, SLOT(infotextclr()));
  ainfoclr->setEnabled(false);
  popupmenu->addAction(ainfoclr);
  ainfomove = new QAction(tr("Infotext verschieben","PopupMenue"), popupmenu);
  connect(ainfomove, SIGNAL(triggered()), this, SLOT(infotextmove()));
  ainfomove->setEnabled(false);
  popupmenu->addAction(ainfomove);
  popupmenu->addSeparator();
  amarker2info = new QAction(tr("Marker zu Infotext","PopupMenue"), popupmenu);
  connect(amarker2info, SIGNAL(triggered()), this, SLOT(fmarker2info()));
  popupmenu->addAction(amarker2info);
  amarker2info->setEnabled(false);
  amarkerclr = new QAction(tr("Marker loeschen","PopupMenue"), popupmenu);
  connect(amarkerclr, SIGNAL(triggered()), this, SLOT(fmarkerclr()));
  popupmenu->addAction(amarkerclr);
  amarkerclr->setEnabled(false);


  //setBackgroundRole(QPalette::Dark);
  setAutoFillBackground(true);
  setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  setFocusPolicy(Qt::StrongFocus);
  rubberBandIsShown = false;
  
  zoomInButton = new QPushButton("+",this);
//  zoomInButton->adjustSize();
  zoomInButton->setGeometry(0,0,20,20);
  connect(zoomInButton, SIGNAL(clicked()), this, SLOT(zoomIn()));
  
  zoomOutButton = new QPushButton("-",this);
  zoomOutButton->setGeometry(20,0,20,20);
  connect(zoomOutButton, SIGNAL(clicked()), this, SLOT(zoomOut()));
  klsfwversion="";
  winfomessung.clear();
  binfotext = false;
  bmarker=false;
  kmarker=-1;
  bmarkermove=false;
}

WDisplay::~WDisplay(){
  //zoomStack.clear();
}

void WDisplay::setPlotSettings(const PlotSettings &settings){
  zoomStack.clear();
  klsfwversion.sprintf("%i.%02i",settings.klsfwversion/100, settings.klsfwversion -((settings.klsfwversion/100)*100));
  zoomStack.append(settings);
  curZoom = 0;
  zoomInButton->hide();
  zoomOutButton->hide();
  refreshPixmap();
}

void WDisplay::zoomOut(){
  if(curZoom > 0){
    --curZoom;
    zoomOutButton->setEnabled(curZoom > 0);
    zoomInButton->setEnabled(true);
    zoomInButton->show();
    bmarkermove=true;
    refreshPixmap();
  }
}

void WDisplay::zoomIn(){
  if(curZoom < zoomStack.count() -1){
    ++curZoom;
    zoomInButton->setEnabled(curZoom < zoomStack.count() -1);
    zoomOutButton->setEnabled(true);
    zoomOutButton->show();
    bmarkermove=true;
    refreshPixmap();
  }
}

void WDisplay::setCurveData(int id, const QVector<QPointF> &data, const QString &sugate){
  curveUgate[id] = sugate;
  curveMap[id] = data;
  idmarker = id;
  dmarker = QPointF(0.0,0.0);
  refreshPixmap();
}

void WDisplay::setInfomessung(QStringList &wim){
  winfomessung = wim;
  refreshPixmap();
}


void WDisplay::clearCurve(int id){
  curveUgate.remove(id);
  curveMap.remove(id);
  refreshPixmap();
}

QSize WDisplay::minimumSizeHint() const{
  return QSize(6 * Margin, 4 * Margin);
}

QSize WDisplay::sizeHint() const{
  return QSize(12 * Margin, 8 * Margin);
}

void WDisplay::paintEvent(QPaintEvent *){
  QStylePainter painter(this);
  painter.drawPixmap(0,0,pixmap);
  if(rubberBandIsShown){
    painter.setPen(Qt::blue);
    //painter.setPen(palette().light().color());
    painter.drawRect(rubberBandRect.normalized().adjusted(0,0,-1,-1));
  }
  if(hasFocus()){
    QStyleOptionFocusRect option;
    option.initFrom(this);
    option.backgroundColor = palette().dark().color();
    painter.drawPrimitive(QStyle::PE_FrameFocusRect, option);
  }
}

void WDisplay::resizeEvent(QResizeEvent *){
  refreshPixmap();
}

void WDisplay::mousePressEvent(QMouseEvent *event){
  //Mausposition merken
  QPoint position=event->pos();
  //qDebug()<<"position:"<<position;
  QRect rect(Margin, Margin, width()-2*Margin, height()-2*Margin);
  if(event->button() == Qt::LeftButton){
    embutton = eblinks;
    if(binfomove == true){
      posinfotext = position;
      //qDebug()<<"mousePressEvent Left posinfotext:"<<posinfotext;
      refreshPixmap();
    }else{
      //qDebug("mousePressEvent(QMouseEvent *event)");
      if(rect.contains(event->pos())){
        rubberBandIsShown = true;
        rubberBandRect.setTopLeft(event->pos());
        rubberBandRect.setBottomRight(event->pos());
        updateRubberBandRegion();
        setCursor(Qt::CrossCursor);
      }
    }
  }
  //PopUp Menu aktivieren
  if(event->button() == Qt::RightButton){
    embutton = ebrechts;
    //an der Cursorposition
    popupmenu->popup(QCursor::pos());
    //Position merken fuer Infotext
    if(!binfotext)posinfotext = position;
    //qDebug()<<"mousePressEvent Right posinfotext:"<<posinfotext;
  }
  if(event->button() == Qt::MidButton){
    embutton = ebmitte;
    setCursor(Qt::PointingHandCursor);
    posmarker = position;
    bmarker = true;
    amarker2info->setEnabled(true);
    amarkerclr->setEnabled(true);
    refreshPixmap();
  }
}

void WDisplay::mouseMoveEvent(QMouseEvent *event){
  switch(embutton){
  case eblinks:
    //Infotext verschieben
    if(binfomove){
      //Position der Maus holen
      posinfotext = event->pos();
      //qDebug()<<"mouseMoveEvent posinfotext:"<<posinfotext;
      //Plot neu zeichnen
      refreshPixmap();
    }
    if(rubberBandIsShown){
      updateRubberBandRegion();
      rubberBandRect.setBottomRight(event->pos());
      updateRubberBandRegion();
    }
    break;
  case ebmitte:
    if(bmarker){
      posmarker = event->pos();
      refreshPixmap();
    }
    break;
  case ebrechts:
    break;
  case ebkein:
    break;
  }
}

void WDisplay::mouseReleaseEvent(QMouseEvent *event){
  //Linker Mausbutton wieder losgelassen
  embutton = ebkein;
  if(event->button() == Qt::LeftButton && binfomove){
    binfomove = false;
    //Kursor normal
    unsetCursor();
    //Plot neu zeichen
    refreshPixmap();
  }
  if((event->button() == Qt::LeftButton) && rubberBandIsShown){
    rubberBandIsShown = false;
    updateRubberBandRegion();
    unsetCursor();
    
    QRect rect= rubberBandRect.normalized();
    if(rect.width() < 4 || rect.height() < 4) return;
    rect.translate(-Margin, -Margin);
    
    PlotSettings prevSettings = zoomStack[curZoom];
    PlotSettings settings = prevSettings;
    double dx = prevSettings.spanX() / (width() -2*Margin);
    double dy = prevSettings.spanY() / (height() -2*Margin);
    settings.minX = prevSettings.minX + dx * rect.left();
    settings.maxX = prevSettings.minX + dx * rect.right();
    settings.minY = prevSettings.maxY - dy * rect.bottom();
    settings.maxY = prevSettings.maxY - dy * rect.top();
    settings.version = prevSettings.version;
    settings.verdatum = prevSettings.verdatum;
    settings.adjust();
    
    zoomStack.resize(curZoom +1);
    zoomStack.append(settings);
    zoomIn();
  }
  if(event->button() == Qt::MidButton){
    unsetCursor();
    bmarker=false;
    refreshPixmap();
  }
}

void WDisplay::keyPressEvent(QKeyEvent *event){
  switch(event->key()){
  case Qt::Key_Plus:
    zoomIn();
    break;
  case Qt::Key_Minus:
    zoomOut();
    break;
  case Qt::Key_Left:
    zoomStack[curZoom].scroll(-1,0);
    refreshPixmap();
    break;
  case Qt::Key_Right:
    zoomStack[curZoom].scroll(+1,0);
    refreshPixmap();
    break;
  case Qt::Key_Down:
    zoomStack[curZoom].scroll(0,-1);
    refreshPixmap();
    break;
  case Qt::Key_Up:
    zoomStack[curZoom].scroll(0,+1);
    refreshPixmap();
    break;
  default:
    QWidget::keyPressEvent(event);
  }
  bmarkermove=true;
}

void WDisplay::wheelEvent(QWheelEvent *event){
  int numDegrees = event->delta() / 8;
  int numTicks = numDegrees / 15;
  
  if(event->orientation() == Qt::Horizontal){
    zoomStack[curZoom].scroll(numTicks,0);
  }else{
    zoomStack[curZoom].scroll(0,numTicks);
  }
  bmarkermove=true;
  refreshPixmap();
}

void WDisplay::updateRubberBandRegion(){
  QRect rect=rubberBandRect.normalized();
  update(rect.left(), rect.top(), rect.width(), 1);
  update(rect.left(), rect.top(), 1, rect.height());
  update(rect.left(), rect.bottom(), rect.width(), 1);
  update(rect.right(), rect.top(), 1, rect.height());
}

void WDisplay::refreshPixmap(){
  PlotSettings settings = zoomStack[curZoom];
  
  pixmap = QPixmap(size());
  pixmap.fill(settings.colorhintergrund);
  QPainter painter(&pixmap);
  painter.initFrom(this);
  drawGrid(&painter);
  drawCurves(&painter);
  update();
}

void WDisplay::drawGrid(QPainter *painter){
  
  PlotSettings settings = zoomStack[curZoom];
  
  QRect rect(Margin, Margin, width() - 2 * Margin, height() - 2 * Margin);
  if(!rect.isValid())return;
  int yt;
  bool bxv=false;
  QString qs;

  //Breite kleiner 600 Pixel X-Beschriftung versetzt oder Zahl > 1000
  for(int i=0; i<= settings.numXTicks; ++i){
    //int x=rect.left() + (i*(rect.width()-1)/settings.numXTicks);
    double label = settings.minX + (i*settings.spanX() / settings.numXTicks);
    if(abs(label) > 1000.0 or width() < 600){
      bxv=true;
    }
  }
  for(int i=0; i<= settings.numXTicks; ++i){
    int x=rect.left() + (i*(rect.width()-1)/settings.numXTicks);
    double label = settings.minX + (i*settings.spanX() / settings.numXTicks);
    if(label < 0.0001 and label > -0.0001)label=0.0;
    painter->setPen(settings.pengitter);
    painter->drawLine(x, rect.top(), x, rect.bottom());
    painter->setPen(settings.pensolidline);
    painter->drawLine(x, rect.bottom(), x, rect.bottom() + 5);
    if(bxv){
      if(i%2){yt = 17;}else{yt = 5;}
    }else{
      yt = 5;
    }
    painter->drawText(x-50, rect.bottom()+yt,100,15,Qt::AlignHCenter | Qt::AlignTop, QString::number(label));
  }
  for(int j=0; j<= settings.numYTicks; ++j){
    int y=rect.bottom() - (j*(rect.height()-1)/settings.numYTicks);
    double label = settings.minY + (j*settings.spanY() / settings.numYTicks);
    if(label < 0.0001 and label > -0.0001)label=0.0;
    painter->setPen(settings.pengitter);
    painter->drawLine(rect.left(), y, rect.right(), y);
    painter->setPen(settings.pensolidline);
    painter->drawLine(rect.left()-5, y, rect.left(), y);
    painter->drawText(rect.left()-Margin,y-10,Margin-5,20,Qt::AlignRight | Qt::AlignVCenter, QString::number(label));
  }
  //Y-Achse Bezeichnung
  painter->setPen(settings.pensolidline);
  painter->drawText(rect.left()-Margin/2-20,rect.top()-Margin/2,100,20,Qt::AlignLeft | Qt::AlignVCenter, settings.yeinheit);
  if(!dmarker.isNull()){
    qs.sprintf("%0.2f",dmarker.y()); qs+=settings.ymasseinheit;
    painter->drawText(rect.left()-Margin/2+30,rect.top()-Margin/2,100,20,Qt::AlignLeft | Qt::AlignVCenter, qs);
  }
  //X-Achse Bezeichnung
  int xpos;
  if(settings.htyp == ediode){
    xpos=rect.right()-Margin/2-80;
  }else{
    xpos=rect.right()-Margin/2-50;
  }
  painter->drawText(xpos,rect.bottom()+Margin/2,100,20,Qt::AlignLeft | Qt::AlignVCenter, settings.xeinheit);
  if(!dmarker.isNull()){
    qs.sprintf("%0.2f",dmarker.x()); qs+=settings.xmasseinheit;
    painter->drawText(rect.right()-Margin/2,rect.bottom()+Margin/2,100,20,Qt::AlignLeft | Qt::AlignVCenter, qs);
  }
  //3. Achse Bezeichnung
  painter->drawText(rect.right(),rect.top()-Margin/2,100,20,Qt::AlignLeft | Qt::AlignVCenter, settings.zeinheit);
  painter->drawRect(rect.adjusted(0,0,-1,-1));

  //Plot Beschreibung, Caption
  painter->setPen(settings.pensolidline);
  painter->drawText(rect.left(),rect.top()-Margin/2,rect.width(),20,Qt::AlignCenter | Qt::AlignVCenter, settings.caption);

  //Plot Ueberschrift
  painter->setPen(settings.penueberschrift);
  painter->setFont( QFont("Sans", settings.ueberschriftsize));
  painter->drawText(rect.left(),rect.top()-Margin/2-20,rect.width(),25,Qt::AlignLeft | Qt::AlignVCenter, settings.ueberschrift);
}

void WDisplay::drawCurves(QPainter *painter){
  QString qs;
  double x, y, ym=0.0, mx, my, dx, dy;
  int k, l, m1, m2;
  
  PlotSettings settings = zoomStack[curZoom];
  
  QRect rectm(Margin, Margin, width() - 2 * Margin, height() - 2 * Margin);
  QRect rect(rectm);
  
  if(!rect.isValid())return;
  
  painter->setClipRect(rect.adjusted(1,1,-1,-1));
  painter->setFont( QFont("Sans", 10));
  QMapIterator<int, QVector<QPointF> > ih(curveMapH);
  QMapIterator<int, QString > jh(curveUgateH);
  while(ih.hasNext()){
    ih.next();
    int idh = ih.key();
    const QVector<QPointF> &datah = ih.value();
    QPolygonF polyline(datah.count());
    for(int kh=0; kh<datah.count(); ++kh){
      double dxh = datah[kh].x() - settings.minX;
      double dyh = datah[kh].y() - settings.minY;
      dyh *= -1.0;
      x= rect.left() + (dxh * (rect.width()-1) / settings.spanX());
      y= rect.bottom() + (dyh * (rect.height()-1) / settings.spanY());
      if(abs(rect.width()+Margin-x) < 10.0){
        ym = y;
      }
      polyline[kh]=QPointF(x,y);
      //qDebug("dx %f",dx);
    }
    //if(jh.hasNext()){
      jh.next();
      qs.sprintf("%i:",idh+1);
      qs = qs + jh.value();
    //}else{
    //  qs= "";
    //}
//    painter->setPen(colorForIds[uint(id) % 6]);
    painter->setPen(settings.penhlinie);
    painter->drawPolyline(polyline);
    //painter->setClipRect(rect.adjusted(0,0,Margin,0));
    painter->setClipRect(rect.adjusted(0,Margin*-1,Margin,0));
    //painter->setClipRect(rect.adjusted(0,0,Margin,0));
    if(ym < Margin)ym = Margin;
    //Schriftgroesse rechts neben der Linie einstellen
    QFont font = painter->font();
    font.setPixelSize(settings.liniefontsize());
    painter->setFont(font);
    painter->drawText(rect.width()+Margin+4,int(ym)-10,47,20,Qt::AlignLeft | Qt::AlignVCenter, qs);
    painter->setClipRect(rect);
  }
  rect = rectm;
  painter->setClipRect(rect.adjusted(1,1,-1,-1));
  QMapIterator<int, QVector<QPointF> > i(curveMap);
  QMapIterator<int, QString > j(curveUgate);
  while(i.hasNext()){
    m2=10000;
    bool bpunkt=false;
    i.next();
    int id = i.key();
    const QVector<QPointF> &data = i.value();
    QPolygonF polyline(data.count());
    //neue Markerposition berechnen mit kmarker
    if(bmarkermove and (kmarker >= 0)){
      dx = data[kmarker].x() - settings.minX;
      dy = data[kmarker].y() - settings.minY;
      dy *= -1.0;
      x= rect.left() + (dx * (rect.width()-1) / settings.spanX());
      y= rect.bottom() + (dy * (rect.height()-1) / settings.spanY());
      if(abs(rect.width()+Margin-x) < 10.0){
        ym = y;
      }
      posdmarker.setX(x);
      posdmarker.setY(y);
      bmarkermove = false;
    }
    //qDebug("data.count:%i",data.count());
    for(k=0; k<data.count(); ++k){
      dx = data[k].x() - settings.minX;
      dy = data[k].y() - settings.minY;
      dy *= -1.0;
      x= rect.left() + (dx * (rect.width()-1) / settings.spanX());
      y= rect.bottom() + (dy * (rect.height()-1) / settings.spanY());
      ym = y;//Bei Ausgangskurve, Position der Kurvennummer
      //neue Markerposition berechnen
      if(bmarker and !bpunkt){
        m1= abs(posmarker.x() - x);
        if(m1<m2){
          m2=m1;
          if(m1<2){
            mx = data[k].x();
            my = data[k].y();
            posdmarker.setX(x);
            posdmarker.setY(y);
            QPointF d(mx,my);
            dmarker=d;
            bpunkt = true;
            kmarker = k;
          }
        }
      }
      polyline[k]=QPointF(x,y);
    }
    j.next();
    qs.sprintf("%i:",id+1);//Kurvennummer
    qs = qs + j.value();//Wert dazu
    painter->setPen(settings.penlinie);//Linie setzen Color usw.
    painter->drawPolyline(polyline);//Linie zeichnen
    painter->setClipRect(rect.adjusted(0,Margin*-1,Margin,0));
    //painter->setClipRect(rect.adjusted(0,0,Margin,0));
    if(ym < Margin)ym = Margin;
    //Schriftgroesse rechts neben der Linie einstellen
    QFont font = painter->font();
    font.setPixelSize(settings.liniefontsize());
    painter->setFont(font);
    //Bei Diode keine seitliche Beschriftung
    if(settings.htyp != ediode)painter->drawText(rect.width()+Margin+4,int(ym)-10,47,20,Qt::AlignLeft | Qt::AlignVCenter, qs);
    //Infos aus Textedit mit einfügen
    if(binfotext and winfomessung.count() > 0){
      //Y Position merken
      k=posinfotext.y();
      painter->setPen(settings.pensolidline);
      QFont fi = painter->font();
      fi.setPixelSize(settings.getlfontsize());
      painter->setFont(fi);
      for(int i=0; i<winfomessung.count();i++){
        qs=winfomessung.at(i);
        painter->drawText(posinfotext.x(),k,200,20,Qt::AlignLeft | Qt::AlignVCenter, qs);
        l=settings.getlfontsize()/3;
        k+=settings.getlfontsize()+l;
      }
    }
    if(!posdmarker.isNull() and idmarker == id){
      //Marker zeichnen
      painter->setPen(Qt::darkRed);
      painter->drawLine(posdmarker.x(),posdmarker.y(),posdmarker.x()-3,posdmarker.y()-10);
      painter->drawLine(posdmarker.x(),posdmarker.y(),posdmarker.x()+3,posdmarker.y()-10);
      painter->drawLine(posdmarker.x()-3,posdmarker.y()-10,posdmarker.x()+3,posdmarker.y()-10);
    }
    painter->setClipRect(rect);
  }
}

void WDisplay::loeschenAlleHKurven(){
  curveUgateH.clear();
  binfomove = false;
  binfotext = false;
  winfomessung.clear();
  ainfoset->setEnabled(true);
  ainfoclr->setEnabled(false);
  ainfoclr->setEnabled(false);
  curveMapH.clear();
  refreshPixmap();
}

void WDisplay::loeschenAlleKurven(){
  curveUgate.clear();
  curveMap.clear();
  binfomove = false;
  binfotext = false;
  ainfoset->setEnabled(true);
  ainfoclr->setEnabled(false);
  ainfoclr->setEnabled(false);
  winfomessung.clear();
  dmarker = QPoint(0,0);
  posdmarker = QPoint(0,0);
  kmarker=-1;
  amarker2info->setEnabled(false);
  amarkerclr->setEnabled(false);
  refreshPixmap();
}

void WDisplay::Kurven2Hintergrund(){
  curveUgateH = curveUgate;
  curveMapH = curveMap;
  binfomove = false;
  binfotext = false;
  winfomessung.clear();
  ainfoset->setEnabled(true);
  ainfoclr->setEnabled(false);
  ainfoclr->setEnabled(false);
  refreshPixmap();
}


void WDisplay::bildspeichern(QString str)
{
  QString qs, qs1;
  QString s=str;
  //QFile f;
  PlotSettings settings = zoomStack[curZoom];

  QSettings klssettings("AFU", "kschreiber");
  QDir imgpath;
  //Dateiname uebernommen vom "Caption" der Wobbelkurve
  if(!s.contains(".png"))s+=".png";
  imgpath.setPath(klssettings.value("imgpath", QDir::homePath()).toString());
#ifdef Q_WS_WIN
  s = QFileDialog::getSaveFileName(this, tr("Bild Speichern","Grafik Filedialog"),
                                   imgpath.filePath(s),"PNG (*.png *.PNG);;BMP (*.bmp *.BMP)", &qs1);
#else
  s = QFileDialog::getSaveFileName(this, tr("Bild Speichern","Grafik Filedialog"),
                                   imgpath.filePath(s),"PNG (*.png *.PNG);;BMP (*.bmp *.BMP);;JPG (*.jpg *.JPG)", &qs1);
#endif
  if(!s.isEmpty()){
    //qDebug("save");
    //Datei ueberpruefen ob Sufix vorhanden
    if(!s.contains(".")){
      if(qs1.contains("PNG"))s += ".png";
      if(qs1.contains("BMP"))s += ".bmp";
#ifdef Q_OS_LINUX
      if(qs1.contains("JPG"))s += ".jpg";
#endif
    }
/*
    pixmap = QPixmap(width(),height()+15);
    pixmap.fill(settings.colorhintergrund);
    QPainter painter(&pixmap);
    painter.initFrom(this);
    drawGrid(&painter);
    drawCurves(&painter);
*/

    QPixmap pixb = QPixmap(width(),height()+15);
    pixb.fill(settings.colorhintergrund);
    QPainter p;
    p.begin(&pixb);//Beginn der Zeichnungen im Pixma
    QFont font=p.font();
    font.setPixelSize(10);
    p.setFont(font);
    p.setPen(settings.pensolidline);
    QString sdt;
    QDateTime dtime=QDateTime::currentDateTime();
    QString swname;
#ifdef Q_WS_WIN
    swname ="KLSwin";
#else
    swname ="KLSlin";
#endif
    //Datum und Zeit eintragen
    sdt=dtime.toString("d.MMMM yyyy hh:mm");
    qs=tr("%1").arg(sdt);
    p.drawText(pixb.rect().left()+10, pixb.rect().bottom()-20, 100, 20, Qt::AlignLeft | Qt::AlignVCenter, qs);
    //FW-Version eintragen
    qs1.sprintf("FW:%i.%02i",settings.klsfwversion/100, settings.klsfwversion -((settings.klsfwversion/100)*100));
    //Name PC-SW, SW-Version, FW-Version
    qs=tr("%1-%2 %3, (c)DL4JAL").arg(swname).arg(settings.version).arg(qs1);
    p.drawText(pixb.rect().right()-200, pixb.rect().bottom()-20, 180, 20, Qt::AlignRight | Qt::AlignVCenter, qs);
    drawGrid(&p);//Beschriftung Wobbelfenster erzeugen
    drawCurves(&p);//Kurve Wobbelfenster zeichen
    p.end();//Ende der Zeichnungen im Paintfenster
    QImage img(pixb.size(), QImage::Format_RGB32);
    img=pixb.toImage();
    QImageWriter iw(s);
    iw.write(img);
    //qDebug()<<"s:"<<s;
    //Sind Infotexte vorhanden
    if(winfomessung.count() > 0){
      //Dateinamen aendern in Textdatei
      if(!s.contains(".png"))s.replace(".png",".txt");
      if(!s.contains(".bmp"))s.replace(".png",".txt");
      if(!s.contains(".jpg"))s.replace(".png",".txt");
      //Textdatei speichern
      QFile f(s);
      if(f.open(QIODevice::WriteOnly)){
        QTextStream ts(&f);
        for(int i=0; i<winfomessung.count(); i++){
          ts << winfomessung.at(i) << endl;
        }
        f.close();
      }
    }
    imgpath.setPath(s);
    s=imgpath.dirName();//Filename ermitteln
    qs=imgpath.absolutePath();//Path mit Filename
    qs.remove(s);//Filename ais Path entfernen
    klssettings.setValue("imgpath",qs);
    //qDebug()<<qs;
  }
}

void WDisplay::infotextset(){
  if(winfomessung.count()>0){
    binfotext = true;
    ainfoset->setEnabled(false);
    ainfoclr->setEnabled(true);
    ainfomove->setEnabled(true);
    refreshPixmap();
    repaint();
  }
}

void WDisplay::infotextclr(){
  binfotext = false;
  ainfoset->setEnabled(true);
  ainfoclr->setEnabled(false);
  ainfomove->setEnabled(false);
  refreshPixmap();
  repaint();
}

void WDisplay::infotextmove(){
  binfomove = true;
  setCursor(Qt::OpenHandCursor);
}

void WDisplay::fmarker2info(){
  emit sendmwert(dmarker);
}

void WDisplay::fmarkerclr(){
  dmarker = QPointF(0.0,0.0);
  posdmarker = QPoint(0,0);
  bmarker = false;
  kmarker = -1;
  amarker2info->setEnabled(false);
  amarkerclr->setEnabled(false);
  refreshPixmap();
}

//=====================================================================================================================

PlotSettings::PlotSettings(){
  /*
  minX = 0.0;
  maxX = 10.0;
  numXTicks = 10;
  
  minY = 0.0;
  maxY = 500.0;
  numYTicks = 10;
  xeinheit = "Uds (V)";
  yeinheit = "Ids (mA)";
  zeinheit = "Ugs (V)";
  ueberschrift = "Ausgangskennlinie";
  */
}

void PlotSettings::setminX(double d){
  minX = d;
}

void PlotSettings::setmaxX(double d){
  maxX = d;
}

void PlotSettings::setminY(double d){
  minY = d;
}

void PlotSettings::setmaxY(double d){
  maxY = d;
}

void PlotSettings::setnumXTicks(int i){
  numXTicks = i;
}

void PlotSettings::setnumYTicks(int i){
  numYTicks = i;
}

void PlotSettings::setxeinheit(const QString &s){
  xeinheit = s;
}

void PlotSettings::setyeinheit(const QString &s){
  yeinheit = s;
}

void PlotSettings::setzeinheit(const QString &s){
  zeinheit = s;
}

void PlotSettings::setueberschrift(const QString &s){
  ueberschrift = s;
}

void PlotSettings::scroll(int dx, int dy){
  
  double stepX = spanX() / numXTicks;
  minX += dx * stepX;
  maxX += dx * stepX;
  
  double stepY = spanY() / numYTicks;
  minY += dy * stepY;
  maxY += dy * stepY;
}

void PlotSettings::adjust(){
  
  adjustAxis(minX, maxX, numXTicks);
  adjustAxis(minY, maxY, numYTicks);
}

void PlotSettings::adjustAxis(double &min, double &max, int &numTicks){
  
  const int MinTicks = 4;
  double grossStep = (max - min) / MinTicks;
  double step = pow(10.0, floor(log10(grossStep)));
  
  if(5*step < grossStep){
    step *= 5;
  }else{
    if(2*step < grossStep)step *= 2;
  }
  
  numTicks = int(ceil(max/step) - floor(min/step));
  if(numTicks < MinTicks)numTicks = MinTicks;
  min = floor(min/step)*step;
  max = ceil(max/step)*step;
}

void PlotSettings::setcolorhintergrund(QColor c){
  colorhintergrund = c;
}

void PlotSettings::setcolorschrift(QColor c){
  penschrift = QPen( c, 0, Qt::SolidLine);
  pensolidline = QPen( c, 0, Qt::SolidLine);
  pendotline = QPen( c, 0, Qt::DotLine);
}

void PlotSettings::setcolorueberschrift(QColor c, int size){
  penueberschrift = QPen( c, 0, Qt::SolidLine);
  ueberschriftsize = size;
}

void PlotSettings::setcolorlinie(QColor c){
  penlinie = QPen( c, 0, Qt::SolidLine);
}

void PlotSettings::setcolorgitter(QColor c){
  pengitter = QPen( c, 0, Qt::DotLine);
}

void PlotSettings::setliniefontsize(int i){
  lfontsize = i;
}

int PlotSettings::liniefontsize(){
  return lfontsize;
}
void PlotSettings::setlfontsize(int i){
  ifontsize = i;
}

int PlotSettings::getlfontsize(){
  return ifontsize;
}


void PlotSettings::setcolorhlinie(QColor c){
  penhlinie = QPen( c, 0, Qt::SolidLine);
}
