/***************************************************************************
                          widgetwobbeln.h  -  description
                             -------------------
    begin                : 15.Mai.2007
    copyright            : (C) 2003 by Andreas Lindenau
    email                : DL4JAL@darc.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef WDISPLAY_H
#define WDISPLAY_H

#include <QtGlobal>
#if QT_VERSION >= 0x050000
#include <QtWidgets>
#else
#include <QtGui>
#endif
#include <QtCore>
  
//class QPen;

enum ehalbleitertyp {enpn, epnp, enmosfet, epmosfet, enjfet, epjfet, ediode};
enum ekurventyp {eausgang, eeingang};
enum emousebutton {ebkein, eblinks, ebmitte, ebrechts};

class PlotSettings
{
  public:
  PlotSettings();
  
  void scroll(int dx, int dy);
  void adjust();
  double spanX() const { return maxX - minX; }
  double spanY() const { return maxY - minY; }

  double minX;
  double maxX;
  double minY;
  double maxY;
  int numXTicks;
  int numYTicks;
  int lfontsize;
  int ifontsize;
  int klsfwversion;
  ehalbleitertyp htyp;

  QString xeinheit;
  QString xmasseinheit;
  QString yeinheit;
  QString ymasseinheit;
  QString zeinheit;
  QString ueberschrift;
  QString version;
  QString verdatum;
  QString caption;

  QColor colorhintergrund;
  QPen penueberschrift;
  int ueberschriftsize;
  QPen penschrift;
  QPen pensolidline;
  QPen pendotline;
  QPen penlinie;
  QPen pengitter;
  QPen penhlinie;
  
  void setminX(double);
  void setmaxX(double);
  void setminY(double);
  void setmaxY(double);
  void setnumXTicks(int);
  void setnumYTicks(int);
  void setxeinheit(const QString &);
  void setyeinheit(const QString &);
  void setzeinheit(const QString &);
  void setueberschrift(const QString &);
  void setcolorhintergrund(QColor);
  void setcolorschrift(QColor);
  void setcolorueberschrift(QColor, int size);
  void setcolorlinie(QColor);
  void setcolorgitter(QColor);
  void setliniefontsize(int);
  int liniefontsize();
  void setlfontsize(int);
  int getlfontsize();
  void setcolorhlinie(QColor);
  
  private:
  static void adjustAxis(double &min, double &max, int &numTicks);

};

class WDisplay : public QWidget
{
  Q_OBJECT
  public:
  WDisplay(QWidget* parent=0);
  ~WDisplay();
  
  void setPlotSettings(const PlotSettings &settings);
  void setCurveData(int id, const QVector<QPointF> &data, const QString &sugate);
  void setInfomessung(QStringList &winfomessung);
  void clearCurve(int id);
  void loeschenAlleKurven();
  void loeschenAlleHKurven();
  void Kurven2Hintergrund();
  QSize minimumSizeHint() const;
  QSize sizeHint() const;
  
  public slots:
  void zoomIn();
  void zoomOut();
  void bildspeichern(QString);

  protected:
  void paintEvent(QPaintEvent *event);
  void resizeEvent(QResizeEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  void mousePressEvent(QMouseEvent *event);
  void keyPressEvent(QKeyEvent *event);
  void wheelEvent(QWheelEvent *event);

  private:
  void updateRubberBandRegion();
  void refreshPixmap();
  void drawGrid(QPainter *painter);
  void drawCurves(QPainter *painter);
  
  enum { Margin = 60 };

  emousebutton embutton;
  
  QPushButton *zoomInButton;
  QPushButton *zoomOutButton;
  //Hintergrundkurven
  QMap<int, QString> curveUgateH; //Fuer Hintergrund
  QMap<int, QVector<QPointF> > curveMapH;
  //Messkurven
  QMap<int, QString> curveUgate; //Fuer Ubasis/Ugate Anzeige
  QMap<int, QVector<QPointF> > curveMap;
  
  
  QVector<PlotSettings> zoomStack;
  int curZoom;
  bool rubberBandIsShown;
  QRect rubberBandRect;
  QPixmap pixmap;
  QString klsfwversion;
  QStringList winfomessung;
  QPoint posinfotext;
  QPoint posmarker;
  QPoint posdmarker;
  QPointF dmarker;
  int kmarker;
  int idmarker;
  bool binfotext;
  bool binfomove;
  bool bmarker;
  bool bmarkermove;

  QAction *ainfoset;
  QAction *ainfoclr;
  QAction *ainfomove;
  QAction *amarker2info;
  QAction *amarkerclr;
  QMenu *popupmenu;

  private slots:
  void infotextset();
  void infotextclr();
  void infotextmove();
  void fmarker2info();
  void fmarkerclr();

  signals:
  void sendmwert(QPointF);

};


#endif
